/*!
*
*   JSBeams : Numerical solution of beam structures in JavaScript
*                      version 0.5.1 (2011-10-15)
*
*                 Copyright (C)  2011  Jan Stransky
*
*      Czech Technical University, Faculty of Civil Engineering,
*  Department of Structural Mechanics, 166 29 Prague, Czech Republic
*
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @fileOverview JavaScript implementation of numerical solution (direct stiffness methos) of beam structures. Modeled structures consists of nodes and beams. Nodes have only geometrical meaning and contains only coordinates, dofs and their values are stored in beams. Different static schemes and methods (clamped vs. pin ends, rigid arms, master x slave nodes etc.) can be modeled by special using of beam DOFs. Requires <a href="http://mech.fsv.cvut.cz/~stransky/software/jsmatrix/">jsmatrix.js</a> loaded before loading this file. For more information see <a href="http://mech.fsv.cvut.cz/~stransky/software/jsbeams/">project homepage</a>.
<br /><br/>JSBeams is a free software distributed under <a href='http://www.gnu.org/licenses/gpl.html'>GNU GPL license</a>.
* @author <a href="http://mech.fsv.cvut.cz/~stransky">Jan Stránský</a>
* @version 0.5.1 (2011-10-15)
*/

// constants for easier manipulation
/**constant*/ var JSB2D_CLAMPED_CLAMPED = 0; // should be 0
/**constant*/ var JSB2D_CC              = 0;
/**constant*/ var JSB2D_CLAMPED_HINGE   = 1;
/**constant*/ var JSB2D_CH              = 1;
/**constant*/ var JSB2D_HINGE_CLAMPED   = 2;
/**constant*/ var JSB2D_HC              = 2;
/**constant*/ var JSB2D_HINGE_HINGE     = 3;
/**constant*/ var JSB2D_HH              = 3;
/**constant*/ var JSB2D_TRUSS           = 3;


/** Returns elements of a that are not in b (Matlab-like function), ret = a - b
* @param {Array} a
* @param {Array} b
* @returns {Array} array of elements of a that are not in b
* @example a = [1,2,3,4,5,6];
* b = [2,5,1,99,54];
* c = setdiff(a,b) // c = [3,4,6]*/
setdiff = function(A,B) {
	var ret = [];
	for (a=0; a<A.length; a++) {
		var noMatch = 1;
		for (var b=0; b<B.length; b++) {
			if (A[a] == B[b]) { noMatch = 0; break; }
		}
		if (noMatch) { ret.push(A[a]); }
	}
	return ret;
}


/** 2d xz plane node implementation. The meaning of current implementation is only geometric, because DOFs itself are stored in beams (for both computational and graphic postprocessing part)
* @class represents nodes in 2d zx plane
* @param {float} x x coordinate of node
* @param {float} y y coordinate of node
* @param {float} z z coordinate of node
* @property {float} x x coordinate
* @property {float} y y coordinate
* @property {float} z z coordinate*/
Node = function(x,y,z) {
	this.x = x || 0.;
	this.y = y || 0.;
	this.z = z || 0.;
}
/**#@+
* @function
* @memberOf Node#
*/
Node.prototype = {
	/** String representation
	* @returns {String} string representation*/
	toString: function() {
		return "Node ( "+this.x+", "+this.y+", "+this.z+" )";
	},

	/** Sets new position to receiver
	* @param {float} x x coordinate of new position
	* @param {float} y y coordinate of new position
	* @param {float} z z coordinate of new position*/
	setXYZ: function(x,y,z) {
		this.x = x; this.y = y; this.z = z;
	},

	/** Sets new position to receiver
	* @param {float} x x coordinate of new position
	* @param {float} z z coordinate of new position*/
	setXZ: function(x,z) {
		this.x = x; this.z = z;
	},

	/** Sets new position to receiver
	* @param {float} y y coordinate of new position
	* @param {float} z z coordinate of new position*/
	setYZ: function(y,z) {
		this.y = y; this.z = z;
	},

	/** Set new x coordinate to receiver
	* @param x x coordinate of new position*/
	setX: function(x) {
		this.x = x;
	},

	/** Set new x coordinate to receiver
	* @param y y coordinate of new position*/
	setY: function(y) {
		this.y = y;
	},

	/** Set new z coordinate to receiver
	* @param z z coordinate of new position*/
	setZ: function(z) {
		this.z = z;
	},

	/** Change position of receiver by given values
	* @param {float} dx length of translation in x direction
	* @param {float} dy length of translation in y direction
	* @param {float} dz length of translation in z direction*/
	translate: function(dx,dy,dz) {
		this.x += dx;
		this.y += dy;
		this.z += dz;
	},

	/** Change position of receiver by given values
	* @param {float} dx length of translation in x direction
	* @param {float} dz length of translation in z direction*/
	translateXZ: function(dx,dz) {
		this.x += dx;
		this.z += dz;
	},

	/** Change position of receiver by given values
	* @param {float} dy length of translation in y direction
	* @param {float} dz length of translation in z direction*/
	translateYZ: function(dy,dz) {
		this.y += dy;
		this.z += dz;
	},

	/** Change horizontal position of receiver by given values
	* @param {float} dx length of translation in x direction*/
	translateX: function(dx) {
		this.x += dx;
	},

	/** Change horizontal position of receiver by given values
	* @param {float} dy length of translation in y direction*/
	translateY: function(dy) {
		this.y += dy;
	},

	/** Change vertical position of receiver by given values
	* @param {float} dz length of translation in z direction*/
	translateZ: function(dz) {
		this.z += dz;
	},

	/** Returns difference between receiver and given node
	* @param {Node} n node to be substracted
	* @returns {Node} difference this-node*/
	sub: function(n) {
		return new Node(this.x-n.x,this.y-n.y,this.z-n.z);
	},

	/** Returns sum of receiver and given node
	* @param {Node} n node to be substracted
	* @returns {Node} difference this+node*/
	add: function(n) {
		return new Node(this.x+n.x,this.y+x.y,this.z+n.z);
	},

	/** Returns receiver multiplied by given number
	* @param {float} f multiplicator
	* @returns {Node} product of multiplication f*this*/
	mulf: function(f) {
		return new Node(f*this.x,f*this.y,f*this.z);
	},

	/**  Returns cross product of receiver and given node
	* @param {Node} n node to be crossed
	* @returns {Node} cross product this x node*/
	cross: function(n) {
		return new Node(this.y*n.z-this.z*n.y, this.z*n.x-this.x*n.z, this.x*n.y-this.y*n.x);
	},

	/**  Returns dot product of receiver and given node
	* @param {Node} n node to be crossed
	* @returns {float} dot product this . node*/
	dot: function(n) {
		return this.x*n.x + this.y*n.y + this.z*n.z;
	},

	/**  Returns square norm of receiver this.dot(this)
	* @returns {float} squared norm of receiver*/
	squaredNorm: function() {
		return this.dot(this);
	},

	/**  Returns (Eucleidian) norm of receiver sqrt(this.dit(this))
	* @returns {float} norm of receiver*/
	norm: function() {
		return Math.sqrt(this.dot(this));
	},
	/**#@-*/
}
/** Constructor, see {@link Node} for input parameters description
* @returns {Node} new Node object*/
Node.create = function(x,y,z) {
	return new Node(x,y,z);
}




/** 2d beam implementation
* @class represents structural 2d beam with 3 degrees of freedom in each node
* @param {Node} node1 1st node
* @param {Node} node2 2nd node
* @param {Object} [params={}] object of parameters in format {param1:val1,param2:val2}, i.e. beam=Beam2d(n1,n2,{dofs:[1,2,3,4],ea:4}); 
* @param {int} [params.type=JSB2D_CC] type of beam. JSB2D_CC for both ends clamped, JSB2D_HC for hinge-clamped, JSB2D_CH for clamped-hinge, JSB2D_HH for hinge-hinge (truss)
* @param {[ints]} [params.dofs=[0,1,2,3,4,5]] degrees of freedom on both ends. number (6,5,4) should correscpond to this.type
* @param {float} [params.ei=1.] bending stiffness [Nm2]
* @param {float} [params.ea=1.] normal stiffness [N]
* @param {bool} [params.hasBodyLoad=false] if some body (continuous force or temperature) load is present
* @param {float} [params.fzloc=0.] constant continuous load parallel to local z coordinate [N/m]
* @param {float} [params.fxloc=0.] constant continuous load parallel to local x coordinate [N/m]
* @param {float} [params.mu=1.] length mass [kg/m]
* @param {float} [params.dT=0.] positiv temperature change (=warming) of beam center line [K]
* @param {float} [params.dTh=0.] (temperature of bottom fibers - temperature of top fibers)/height ((Tb-Tt)/h) [K/m]
* @param {float} [params.alpha=12e-6] thermal dillatation coefficient [K-1]
* @property {Node} node1 1st node
* @property {Node} node2 2nd node
* @property {[ints]} dofs degrees of freedom on both ends
* @property {bool} hasBodyLoad if some body (continuous force or temperature) load is present
* @property {float} ei bending stiffness [Nm2]
* @property {float} ea normal stiffness [N]
* @property {float} fzloc constant continuous load parallel to local z coordinate [N/m]
* @property {float} fxloc constant continuous load parallel to local x coordinate [N/m]
* @property {float} mu length mass [kg/m]
* @property {float} dT positive temperature change (=warming) of beam center line [K]
* @property {float} dTh (temperature of bottom fibers - temperature of top fibers)/height ((Tb-Tt)/h) [K/m]
* @property {float} alpha thermal dillatation coefficient [K-1]
* @property {Matrix} kloc stiffness matrix in local coordinate system, computed by setStiffAndTrsfMats method
* @property {Matrix} t transformation matrix (from global to local), computed by setStiffAndTrsfMats method
* @property {Matrix} tt transposed transformation matrix (from local to global), computed by setStiffAndTrsfMats method
* @property {Matrix} mloc mass matrix in local coordinate system, computed by setMassMats method
* @property {Matrix} m mass matrix in global coordinate system, computed by setMassMats method
* @property {Vector} load vector of end forces
* @property {Vector} dspl vector of end displacements
* @property {[Vectors]} eigShapes array of eigenshapes
* @property {float} len length of the beam
* @property {float} sin sin of node1->node2 vector
* @property {float} cos cos of node1->node2 vector*/
// example?
Beam2d = function(node1,node2,params) {
	var params = params==undefined? {} : params;
	this.node1 = node1 || null;
	this.node2 = node2 || null;
	this.type = params.type || JSB2D_CC;
	this.dofs = params.dofs || [0,1,2,3,4,5];
	this.ei = params.ei || 1.;
	this.ea = params.ea || 1.;
	this.hasBodyLoad = params.hasBodyLoad || false;
	this.fzloc = params.fzloc || 0.;
	this.fxloc = params.fxloc || 0.;
	this.mu = params.mu || 1.;
	this.dT = params.dT || 0.;
	this.dTh = params.dTh || 0.;
	this.alpha = params.alpha || 12e-6;
	this.kloc = null;
	this.a    = null;
	this.b    = null;
	this.kaa  = null;
	this.kab  = null;
	this.kba  = null;
	this.kbb  = null;
	this.t    = new Matrix();
	this.tt   = new Matrix();
	this.mloc = new Matrix();
	this.m    = new Matrix();
	this.load = Vector.Zeros(6);
	this.loadCC = Vector.Zeros(6);
	this.dspl = Vector.Zeros(6);
	this.eigShapes = [];
	this.len = null;
	this.sin = null;
	this.cos = null;

	this.setGeom();
}
/**#@+
* @function
* @memberOf Beam2d#
*/
Beam2d.prototype = {
	/** Sets geometric parameters of received according to node positions*/
	setGeom: function() {
		var dx = (this.node2.x - this.node1.x);
		var dz = (this.node2.z - this.node1.z);
		this.len = Math.sqrt(dx*dx + dz*dz);
		this.cos = dx/this.len;
		this.sin = dz/this.len;
	},

	/** Sets stiffness matrices in local and global coordinate system according to stored material and crossection parameters and transformation matrix and its transposition according to stored geometry*/
	setStiffAndTrsfMats: function() {
		var l = this.len;
		var eal = this.ea/l;
		var eil = this.ei/l;
		var eil2 = eil/l;
		var eil3 = eil2/l;
		var s = this.sin;
		var c = this.cos;
		// T - from gloal to local, TT - from local to global
		switch (this.type) {
			case JSB2D_HC:
				this.t.fromArray([[c,s,0,0,0],[-s,c,0,0,0],[0,0,c,s,0],[0,0,-s,c,0],[0,0,0,0,1]]);
				this.tt.fromArray([[c,-s,0,0,0],[s,c,0,0,0],[0,0,c,-s,0],[0,0,s,c,0],[0,0,0,0,1]]);
				break;
			case JSB2D_CH:
				this.t.fromArray([[c,s,0,0,0],[-s,c,0,0,0],[0,0,1,0,0],[0,0,0,c,s],[0,0,0,-s,c]]);
				this.tt.fromArray([[c,-s,0,0,0],[s,c,0,0,0],[0,0,1,0,0],[0,0,0,c,-s],[0,0,0,s,c]]);
				break;
			case JSB2D_HH:
				this.t.fromArray([[c,s,0,0],[-s,c,0,0],[0,0,c,s],[0,0,-s,c]]);
				this.tt.fromArray([[c,-s,0,0],[s,c,0,0],[0,0,c,-s],[0,0,s,c]]);
				break;
			default: // c-c
				this.t.fromArray([[c,s,0,0,0,0],[-s,c,0,0,0,0],[0,0,1,0,0,0],[0,0,0,c,s,0],[0,0,0,-s,c,0],[0,0,0,0,0,1]]);
				this.tt.fromArray([[c,-s,0,0,0,0],[s,c,0,0,0,0],[0,0,1,0,0,0],[0,0,0,c,-s,0],[0,0,0,s,c,0],[0,0,0,0,0,1]]);
		}
		var k = Matrix.create([
			[ eal,   0     ,  0     , -eal,   0     ,  0     ],
			[ 0  ,  12*eil3, -6*eil2,  0  , -12*eil3, -6*eil2],
			[ 0  , - 6*eil2,  4*eil ,  0  ,   6*eil2,  2*eil ],
			[-eal,   0     ,  0     ,  eal,   0     ,  0     ],
			[ 0  , -12*eil3,  6*eil2,  0  ,  12*eil3,  6*eil2],
			[ 0  , - 6*eil2,  2*eil ,  0  ,   6*eil2,  4*eil ]]);
		if (this.type) { // type is not c-c
			switch (this.type) {
				case JSB2D_HC: // hinged-clamped
					this.a = [0,1,3,4,5];
					this.b = [2];
					break;
				case JSB2D_CH: // clamped hinge
					this.a = [0,1,2,3,4];
					this.b = [5];
					break;
				case JSB2D_HH: // truss
					this.a = [0,1,3,4];
					this.b = [2,5];
					break;
			}
			this.kaa = k.getsm(this.a,this.a);
			this.kab = k.getsm(this.a,this.b);
			this.kba = k.getsm(this.b,this.a);
			this.kbbinv = k.getsm(this.b,this.b).inv();
			this.kloc = this.kaa.sub(this.kab.mulm(this.kbbinv).mulm(this.kba))
		}
		else { this.kloc = k; }
	},

	/** Computes stiffness matrix in global coordinates system
	* @return Matrix stiffness matrix and corresponding dofs*/
	computeGlobStiffMatrix: function() {
		return this.tt.mulm(this.kloc).mulm(this.t);
	},

	/** Sets mass matrices in local and gloal coordinate system from stored material and geometric parameters*/
	setMassMats: function() {
		var l = this.len;
		var mu = this.mu;
		var a = mu*l/420;
		var s = this.sin;
		var c = this.cos;
		this.Mloc.fromArray([
			[ 140*a, 0     , 0      , 70*a , 0     , 0      ],
			[ 0    , 156*a ,-22*l*a , 0    , 54*a  , 13*a*l ],
			[ 0    ,-22*l*a, 4*a*l*l, 0    ,-13*a*l,-3*a*l*l],
			[ 70*a , 0     , 0      , 140*a, 0     , 0      ],
			[ 0    , 54*a  ,-13*a*l , 0    , 156*a , 22*a*l ],
			[ 0    , 13*a*l,-3*a*l*l, 0    , 22*a*l, 4*a*l*l]]);
		/*this.M.fromArray([
			[ 140*a*c*c + 156*a*s*s,              16*a*c*s, -22*a*l*s,   70*a*c*c + 54*a*s*s,             -16*a*c*s, 13*a*l*s],
			[              16*a*c*s, 156*a*c*c + 140*a*s*s, -22*a*c*l,             -16*a*c*s,   54*a*c*c + 70*a*s*s, 13*a*c*l],
			[             -22*a*l*s,             -22*a*c*l,   4*a*l*l,             -13*a*l*s,             -13*a*c*l, -3*a*l*l],
			[   70*a*c*c + 54*a*s*s,             -16*a*c*s, -13*a*l*s, 140*a*c*c + 156*a*s*s,              16*a*c*s, 22*a*l*s],
			[             -16*a*c*s,   54*a*c*c + 70*a*s*s, -13*a*c*l,              16*a*c*s, 156*a*c*c + 140*a*s*s, 22*a*c*l],
			[              13*a*l*s,              13*a*c*l,  -3*a*l*l,              22*a*l*s,              22*a*c*l,  4*a*l*l]]);*/
	},

	/** Sets geometry, stiffness matrices and "beam" load (end forces induced by continuous load and temperature load)*/
	linearStaticPreproc: function() {
		this.setGeom();
		this.setStiffAndTrsfMats();
		this.load.zero();
		if (this.hasBodyLoad) {
			this.loadCC.fromArray([
				this.ea*this.alpha*this.dT - 0.5*this.fxloc*this.len ,
				-.5*this.fzloc*this.len ,
				this.fzloc*this.len*this.len/12. + this.ei*this.alpha*this.dTh,
				-this.ea*this.alpha*this.dT ,
				-.5*this.fzloc*this.len - 0.5*this.fxloc*this.len ,
				-this.fzloc*this.len*this.len/12. - this.ei*this.alpha*this.dTh ]);
			if (this.type) {
				this.load.zero();
				this.load.setsv(this.a,this.loadCC.getsv(this.a).sub(this.kab.mulm(this.kbbinv).mulv(this.loadCC.getsv(this.b))))
			}
			else {
				this.load = this.loadCC;
			}
		}
	},

	/** Computes end displacement in local coordinate system from given global components and respective end local end forces
	* @param {Vector} dspl vector of beam displacement in globalComponents (if global param is true) or local (if globalComponents param is false) coordinate system
	* @param {bool} [globalComponents=true] if true, dspl is considered in global coordinate system. If false, dspl is considered in local coordinate system*/
	linearStaticPostpro: function(dspl,globalComponents) {
		var globalComponents = globalComponents || true;
		var dspl = globalComponents? this.t.mulv(dspl) : dspl;
		if (this.type) {
			this.dspl.zero();
			this.dspl.setsv(this.a,dspl);
			this.dspl.setsv(this.b,this.kbbinv.mulv(this.loadCC.getsv(this.b).add(this.kba.mulv(dspl))).negated());
			this.load.zero();
			this.load.setsv(this.a,this.kloc.mulv(dspl).add(this.loadCC.getsv(this.a).sub(this.kab.mulm(this.kbbinv).mulv(this.loadCC.getsv(this.b)))))
		}
		else {
			this.dspl = dspl;
			this.load = this.kloc.mulv(this.dspl).add(this.loadCC);
		}
	},

	/** Sets geometry and stiffness and mass matrices*/
	eigenvalueDynamicPreproc: function() {
		this.setGeom();
		this.setStiffAndTrsfMats();
		this.setMassMats();
	},

	/** Compute local components of given eigenshape
	* @param {int} eigMode number of assumed eigenshape - numbering from 0
	* @param {Vector} eigShape eigMode-th eigenshape in global coordinate system*/
	eigenvalueDynamicPostpro: function(eigMode,eigShape) {
		this.eigShapes[eigMode] = this.TT.mulv(eigShape);
	},

	/** Returns middle deflection of receiver due to continuous load (clamped ends are assumed)
	* @returns {float} middle deflection from continuous load*/
	giveMidDeflFromLoad: function() { // local z deflection in the middle of doble-clamped beam from fzloc
		return this.fzloc*this.len*this.len*this.len*this.len / (384*this.ei);
	},

	/** Returns middle deflection of receiver due to end displacement (no load is assumed)
	* @returns {float} middle deflection from end displacement*/
	giveMidDeflFromDspl: function() { // local z deflection in the middle of the beam form nodal dspl and fzloc=0
		return .5*(this.dspl.get(1) + this.dspl.get(4)) + this.len/8*( -this.dspl.get(2) + this.dspl.get(5));
	},

	/** Returns middle deflection of receiver
	* @returns {float} middle deflection*/
	giveMidDefl: function() {
		return this.giveMidDeflFromDspl() + this.giveMidDeflFromLoad();
	},
	
	/** Returns middle deflection of receiver of eigMode-th eigenshape
	* @param {int} eigMode number of asumed eigenshape - numbering from 0
	* @returns {float} middle deflection of eigMode-th eigenshape*/
	giveMidDeflOfEigShape: function(eigMode) { // local z deflection in the middle of the beam of eigMode-th eigenShape
		var s = this.eigShapes[eigMode];
		return .5*(s.get(1) + s.get(4)) + this.len/8*( -s.get(2) + s.get(5));
	},

	/** Returns middle bending of receiver
	* @returns {float} middle deflection*/
	giveMidMoment: function() { // bending moment in the middle of the beam
		return -this.load.get(2) - this.load.get(1)*this.len*.5 - this.fzloc*this.len*this.len*.125;
	},
	/**#@-*/
}

/** Constructor, see {@link Beam2d} for input parameters description
* @returns {Beam2d} new Beam2d object*/
Beam2d.create = function(node1,node2,params) {
	return new Beam2d(node1,node2,params);
}



/** Linear static solver implementation
* @class represents linear static solver
* @param {[Beams]} beams array of beams to include
* @param {[ints]} u array of unsupported dofs
* @param {[ints]} p array of supported dofs
* @param {Vector} nodalLoad vector of external load in nodes (load from beams, i.e. linear load, temperature etc. will be added separately from beams)
* @param {bool} [withPrescribedDspl=true] if there is some nonzero prescribed displacement of support
* @param {bool} [computeReactions=false] compute global reactions or not
* @property {[Beams]} beams array of beams to include
* @property {[ints]} u array of unsupported dofs
* @property {[ints]} p array of supported dofs
* @property {int} nDofs total number of degrees of freedom of solved structure
* @property {Vector} nodalLoad vector of external load in nodes (load from beams, i.e. linear load, temperature etc. will be added separately from beams)
* @property {bool} [withPrescribedDspl=false] if there is some nonzero prescribed displacement of support
* @property {Matrix} K global stiffnesss matrix of whole structure
* @property {Vector} dspl displacement vector
* @property {Vector} beamLoad vector of "beam loads", i.e. end forces from beam continuous load and temperature load
* @property {Vector} load total load (sum od nodalLoad and beamLoad)
* @example TODO*/
LinearStaticSolver = function(beams,u,p,dspl,nodalLoad,withPrescribedDspl,computeReactions) {
	this.beams = beams;
	this.u = u;
	this.p = p;
	this.nDofs = this.u.length + this.p.length;
	this.nodalLoad = nodalLoad==undefined? Vector.Zeros(this.nDofs) : nodalLoad;
	this.withPrescribedDspl = withPrescribedDspl==undefined? false : withPrescribedDspl;
	this.computeReactions = computeReactions==undefined? false : computeReactions;
	this.k = Matrix.Zeros(this.nDofs,this.nDofs);
	this.dspl = dspl==undefined? Vector.Zeros(this.nDofs) : dspl;
	this.beamLoad = Vector.Zeros(this.nDofs);
	this.load = Vector.Zeros(this.nDofs);
}
/**#@+
* @function
* @memberOf LinearStaticSolver#
*/
LinearStaticSolver.prototype = {

	/** Assembles global stiffness matrix of whole structure from stiffness matrices of particular beams*/
	assembly: function() {
		this.k.zero();
		for (var beamNum=0; beamNum<this.beams.length; beamNum++) {
			var beam = this.beams[beamNum];
			var dofs = beam.dofs;
			this.k.incrsm(dofs,dofs,beam.computeGlobStiffMatrix());
		}
	},

	/** Solves unknown displacements and forces*/
	solve: function() {
		this.load = this.nodalLoad.add(this.beamLoad);
		var kuu = this.k.getsm(this.u,this.u);
		var kpu = this.k.getsm(this.p,this.u);
		var fu = this.load.getsv(this.u);
		if (this.withPrescribedDspl) {
			var kup = this.k.getsm(this.u,this.p);
			var kpp = this.k.getsm(this.p,this.p);
			var rp = this.dspl.getsv(this.p);
			this.dspl.setsv(this.u,kuu.solveForRhs(fu.sub(kup.mulv(rp))));
			if (this.computeReactions) { this.load.setsv(this.p,kpu.mulv(this.dspl.getsv(this.u)).add(kpp.mulv(rp))); }
		}
		else {
			this.dspl.setsv(this.u,kuu.solveForRhs(fu));
			if (this.computeReactions) { this.load.setsv(this.p,kpu.mulv(this.dspl.getsv(this.u))); }
		}
	},

	/** Preprocesses all beams (sets their "beam load" and assemble it to global load vecor, set their geometric parameters and stiffness matrices*/
	preprocBeams: function() {
		this.beamLoad.zero()
		for (var beamNum=0; beamNum<this.beams.length; beamNum++) {
			var beam = this.beams[beamNum];
			beam.linearStaticPreproc();
			if (beam.type) {
				this.beamLoad.decrsv(beam.dofs,beam.tt.mulv(beam.load.getsv(beam.a)));
			} else {
				this.beamLoad.decrsv(beam.dofs,beam.tt.mulv(beam.load));
			}
		}
	},

	/** Postprocesses all beams (sets appropriate displacements to them)*/
	updateBeams: function() {
		for (var beamNum=0; beamNum<this.beams.length; beamNum++) {
			var beam = this.beams[beamNum];
			beam.linearStaticPostpro(this.dspl.getsv(beam.dofs));
		}
	},

	/** Gloal function for whole simulation process*/
	up: function(preprocBeams,updateBeams) {
		var preprocBeams = preprocBeams==undefined? true : preprocBeams;
		var updateBeams = updateBeams==undefined? true : updateBeams;
		if (preprocBeams) { this.preprocBeams(); }
		this.assembly();
		this.solve();
		if (updateBeams) { this.updateBeams(); }
	},
	/**#@-*/
}
/** Constructor, see {@link LinearStaticSolver} for input parameters description
* @returns {LinearStaticSolver} new LinearStaticSolver object*/
LinearStaticSolver.create = function(beams,u,p,dspl,nodalLoad,withPrescribedDspl) {
	return new LinearStaticSolver(beams,u,p,dspl,nodalLoad,withPrescribedDspl);
}




/** Eigenvalue dynamic solver implementation
* @class represents eigenvalue dynamic solver
* @param {[Beams]} beams array of beams to include
* @param {[ints]} u array of unsupported dofs
* @param {[ints]} p array of supported dofs
* @param {int} [nEigModes=1] number of first eigenvalues and eigenvectors to be solved
* @param {Vector} [concMass=Vector.Zeros] Vector of concentrated mass for each Dof (default zero Vector)
* @property {[Beams]} beams array of beams to include
* @property {[ints]} u array of unsupported dofs
* @property {[ints]} p array of supported dofs
* @property {int} nDofs total number of degrees of freedom of solved structure
* @property {int} [nEigModes=1] number of first eigenvalues and eigenvectors to be solved
* @property {Vector} [concMass=Vector.Zeros] Vector of concentrated mass for each Dof (default zero Vector)
* @property {Matrix} K global stiffness matrix
* @property {Matrix} M global mass matrix
* @property {[floats]} eigFreqs computed eigenfrequencies
* @property {[Vectors]} eigShapes computed eigenshapes
* @example TODO*/
EigenValueDynamicSolver = function(beams,u,p,nEigModes,concMass) {
	this.beams = beams;
	this.u = u;
	this.p = p;
	this.nDofs = this.u.length + this.p.length;
	this.nEigModes = nEigModes==undefined? 1 : nEigModes;
	this.concMass = concMass==undefined? Vector.Zeros(this.nDofs) : concMass;
	this.K = Matrix.Zeros(this.nDofs,this.nDofs);
	this.M = Matrix.Zeros(this.nDofs,this.nDofs);
	this.eigFreqs = [];
	for (var eigMode=0; eigMode<this.nEigModes; eigMode++) { this.eigFreqs[eigMode] = 0.; }
	this.eigShapes = [];
	for (var eigMode=0; eigMode<this.nEigModes; eigMode++) { this.eigShapes[eigMode] = Vector.Zeros(this.nDofs); }
}
/**#@+
* @function
* @memberOf EigenValueDynamicSolver#
*/
EigenValueDynamicSolver.prototype = {

	/** Assembles global stiffness and mass matrix of whole structure from stiffness and mass matrices of particular beams and concMass attribute*/
	assembly: function() {
		this.K.zero();
		this.M.zero()
		for (var beamNum=0; beamNum<this.beams.length; beamNum++) {
			var beam = this.beams[beamNum];
			var dofs = beam.dofs;
			this.K.incrsm(dofs,dofs,beam.K);
			this.M.incrsm(dofs,dofs,beam.M);
		}
		for (var i=0; i<this.nDofs; i++) { this.M.incr(i,i,this.concMass.get(i)); }
	},

	/** Solves unknown eigenfrequencies and eigenshapes*/
	solve: function() {
		var kuu = this.k.getsm(this.u,this.u);
		var kuu = this.m.getsm(this.u,this.u);
		var results = eigSolve(Kuu,Muu,this.nEigModes);
		for (var eigMode=0; eigMode<this.nEigModes; eigMode++) {
			this.eigFreqs[eigMode] = results[0][eigMode];
			this.eigShapes[eigMode].setsv(this.u,results[1][eigMode]);
		}
	},

	/** Preprocesses all beams (sets their "beam load" and assemble it to global load vecor, set their geometric parameters and stiffness matrices*/
	preprocBeams: function() {
		for (var beamNum = 0; beamNum<this.beams.length; beamNum++) { this.beams[beamNum].eigenvalueDynamicPreproc(); }
	},

	/** Postprocesses all beams (from computed eigenshapes sets appropriate displacements to them)*/
	updateBeams: function() {
		for (var eigMode = 0; eigMode<this.nEigModes; eigMode++) {
			for (var beamNum=0; beamNum<this.beams.length; beamNum++) {
				var beam = this.beams[beamNum];
				beam.eigenvalueDynamicPostpro(eigMode,this.eigShapes[eigMode].getsv(beam.dofs));
			}
		}
	},

	/** Global function for whole simulation process*/
	up: function(preprocBeams,updateBeams) {
		var preprocBeams = preprocBeams==undefined? true : preprocBeams;
		var updateBeams = updateBeams==undefined? true : updateBeams;
		if (preprocBeams) { this.preprocBeams(); }
		this.assembly();
		this.solve();
		if (updateBeams) { this.updateBeams(); }
	},
	/**#@-*/
}
/** Constructor, see {@link EigenValueDynamicSolver} for input parameters description
* @returns {EigenValueDynamicSolver} new EigenValueDynamicSolver object*/
EigenValueDynamicSolver.create = function(beams,u,p,nEigModes,concMass) {
	return new EigenValueDynamicSolver(beams,u,p,nEigModes,concMass);
}










/** Cross section implementation
* @class represents (so far polygonal only) cross section
* @param {[Nodes]} nodes array of nodes to create yz (!!) polygon in anti-clockwise (!!) order
* @param {[[Nodes]]} [holes=[[]]] array of arrays of nodes representing holes in polygon created by nodes
* @property {[Nodes]} nodes array of polygon nodes in yz plane ( this.nodes[0]==this.nodes[this.nodes.length-1] )
* @property {float} [a=null] area
* @property {float} [sy=null] static moment (first moment of mass) with respect to y coordinate (global)
* @property {float} [sz=null] static moment (first moment of mass) with respect to z coordinate (global)
* @property {float} [cy=null] y coordinate of center of mass
* @property {float} [cz=null] z coordinate of center of mass
* @property {float} [iy=null] moment of inertia (second moment of area) with respect to y coordinate (local)
* @property {float} [iz=null] moment of inertia (second moment of area) with respect to z coordinate (local)
* @property {float} [dyz=null] deviation (product) moment of inertia with respect to yz coordinate system
* @property {float} [i1=null] first principal value of moment of inertia (i1>i2)
* @property {float} [i2=null] second principal value of moment of inertia (i1>i2)
* @property {float} [beta=null] angle from global axes to principal inertia orientation
* @property {[Nodes]} [core=null] core of cross section
* @example TODO*/
CrossSection = function(nodes,holes) {
	this.nodes = nodes;
	if (this.nodes[this.nodes.length-1] != this.nodes[0]) { this.nodes.push(this.nodes[0]); }
	this.holes = holes || [[]];
	this.a = null;
	this.sy = null;
	this.sz = null;
	this.cy = null;
	this.cz = null;
	this.iy = null;
	this.iz = null;
	this.dyz = null;
	this.i1 = null;
	this.i2 = null;
	this.beta = null;
	this.core = null;
}
/**#@+
* @function
* @memberOf CrossSection#
*/
CrossSection.prototype = {
	/** Reset receiver (sets all values to null)*/
	reset: function() {
		this.a = null;
		this.sy = null;
		this.sz = null;
		this.cy = null;
		this.cz = null;
		this.iy = null;
		this.iz = null;
		this.dyz = null;
		this.i1 = null;
		this.i2 = null;
		this.beta = null;
		this.convexHull = null;
		this.core = null;
	},

	/** Computes area of receiver
	* @returns {float} area of receiver*/
	computeA: function() {
		var ret = 0.;
		var n0, nL;
		for (var i=0; i<this.nodes.length-1; i++) {
			n0 = this.nodes[i];
			nL = this.nodes[i+1];
			ret += n0.y*nL.z - nL.y*n0.z;
		}
		return .5*ret;
	},

	/** Computes static moment (first order moment of mass) of receiver with respect to y coordinate axis
	* @param {bool} [local=false] if computed with respect to this.cy center of mass or global yz coordinates
	* @returns {float} Sy of receiver*/
	computeSy: function(local) {
		var local = local==undefined? false : local;
		var zc = 0.;
		if (local) {
			if (this.cz==null) { return null; }
			zc = this.cz;
		}
		var ret = 0.;
		var n0, nL, y0, yL, z0, zL;
		for (var i=0; i<this.nodes.length-1; i++) {
			n0 = this.nodes[i];
			nL = this.nodes[i+1];
			y0 = n0.y; yL = nL.y;
			z0 = n0.z-zc; zL = nL.z-zc;
			ret += (zL+z0)*(y0*zL-yL*z0);
		}
		return 1/6.*ret;
	},

	/** Computes static moment (first order moment of mass) of receiver with respect to z coordinate axis
	* @param {bool} [local=false] if computed with respect to this.cz center of mass or global yz coordinates
	* @returns {float} Sz of receiver*/
	computeSz: function(local) {
		var local = local==undefined? false : local;
		var yc = 0.;
		if (local) {
			if (this.cy==null) { return null; }
			yc = this.cy;
		}
		var ret = 0.;
		var n0, nL, y0, yL, z0, zL;
		for (var i=0; i<this.nodes.length-1; i++) {
			n0 = this.nodes[i];
			nL = this.nodes[i+1];
			y0 = n0.y-yc; yL = nL.y-yc;
			z0 = n0.z; zL = nL.z;
			ret += (y0+yL)*(y0*zL-yL*z0);
		}
		return 1/6.*ret;
	},

	/** Computes y coordinate of center of mass of receiver
	* @returns {float} Cy of receiver*/
	computeCy: function() {
		var a = this.a==null? this.computeA() : this.a;
		var sz = this.sz==null? this.computeSz() : this.sz;
		return sz/a;
	},

	/** Computes z coordinate of center of mass of receiver
	* @returns {float} Cz of receiver*/
	computeCz: function() {
		var a = this.a==null? this.computeA() : this.a;
		var sy = this.sy==null? this.computeSy() : this.sy;
		return sy/a;
	},

	/** Computes moment of inertia (second moment of area) of receiver with respect to y coordinate axis
	* @param {bool} [local=true] if computed with respect to this.cy center of mass or global yz coordinates
	* @returns {float} moment of inertia (y) of receiver*/
	computeIy: function(local) {
		var local = local==undefined? true : local;
		var zc = 0.;
		if (local) {
			if (this.cz==null) { return null; }
			zc = this.cz;
		}
		var ret = 0.;
		var n0, nL, y0, yL, z0, zL;
		for (var i=0; i<this.nodes.length-1; i++) {
			n0 = this.nodes[i];
			nL = this.nodes[i+1];
			y0 = n0.y; yL = nL.y;
			z0 = n0.z-zc; zL = nL.z-zc;
			ret += (z0*z0+z0*zL+zL*zL)*(y0*zL-yL*z0);
		}
		return 1/12.*ret;
	},

	/** Computes moment of inertia (second moment of area) of receiver with respect to z coordinate axis
	* @param {bool} [local=true] if computed with respect to this.cz center of mass or global yz coordinates
	* @returns {float} moment of inertia (z) of receiver*/
	computeIz: function(local) {
		var local = local==undefined? true : local;
		var yc = 0.;
		if (local) {
			if (this.cy==null) { return null; }
			yc = this.cy;
		}
		var ret = 0.;
		var n0, nL, y0, yL, z0, zL;
		for (var i=0; i<this.nodes.length-1; i++) {
			n0 = this.nodes[i];
			nL = this.nodes[i+1];
			y0 = n0.y-yc; yL = nL.y-yc;
			z0 = n0.z; zL = nL.z;
			ret += (y0*y0+y0*yL+yL*yL)*(y0*zL-yL*z0);
		}
		return 1/12.*ret;
	},

	/** Computes deviation (prodict) moment of inertia of receiver with respect to yz coordinates
	* @param {bool} [local=true] if computed with respect to this.cy and this.cz (center of mass) or global yz coordinates
	* @returns {float} deviation (product) moment of inertia (yz) of receiver*/
	computeDyz: function(local) {
		var local = local==undefined? true : local;
		var yc = 0.;
		var zc = 0.;
		if (local) {
			if (this.cy==null) { return null; }
			yc = this.cy;
			if (this.cz==null) { return null; }
			zc = this.cz;
		}
		var ret = 0.;
		var n0, nL, y0, yL, z0, zL;
		for (var i=0; i<this.nodes.length-1; i++) {
			n0 = this.nodes[i];
			nL = this.nodes[i+1];
			y0 = n0.y-yc; yL = nL.y-yc;
			z0 = n0.z-zc; zL = nL.z-zc;
			ret += (y0*zL+2*y0*z0+2*yL*zL+yL*z0)*(y0*zL-yL*z0);
		}
		return 1/24.*ret;
	},

	/** Computes polar moment of inertia of receiver with respect to yz coordinate axis
	* @returns {float polar} moment of inertia (yz) of receiver*/
	computeI0: function() {
		var iy = this.iy==null? this.computeIy() : this.iy;
		var iz = this.iz==null? this.computeIz() : this.iz;
		return iy+iz;
	},

	/** Computes moment of inertia tensor of receiver (i.e. [[Iy,-Dyz],[-Dyz,Iz]] )
	* @returns {[[float,float],[float,float]]} moment of inertia tensor (yz) of receiver*/
	computeInertiaTensor: function() {
		var iy = this.iy==null? this.computeIy() : this.iy;
		var iz = this.iz==null? this.computeIz() : this.iz;
		var dyz = this.dyz==null? this.computeDyz() : this.dyz;
		return [[iy,-dyz],[-dyz,iz]];
	},

	/** Computes principal inertia values and angle from global axes to principal inertia orientation
	* @returns {[float,float,float]} principal values and angle of principal direction [I1,I2,angle], I1 > I2 */
	computePrincipalValues: function() {
		var iy = this.iy==null? this.computeIy() : this.iy;
		var iz = this.iz==null? this.computeIz() : this.iz;
		var dyz = this.dyz==null? this.computeDyz() : this.dyz;
		if (iy==iz) { return [iy,iz,0.]; }
		if (dyz == 0.) {
			if (iy>iz) { return [iy,iz,0.]; }
			else { return [iz,iy,.5*Math.PI]; }
		}
		var beta = 0.5*Math.atan(2*dyz/(iz-iy));
		var c = Math.cos(beta)
		var s = Math.sin(beta)
		var i1 = c*c*iy + s*s*iz - 2*c*s*dyz;
		var i2 = s*s*iy + c*c*iz + 2*c*s*dyz;
		if (i1>i2) { return [i1,i2,beta]; }
		else { return [i2,i1,beta+0.5*Math.PI]; }
	},

	/** Computes normal stress at certain point with coordinates x and y
	* @param {float} y y coordinate of desired point
	* @param {float} z z coordinate of desired point
	* @param {float} N normal force
	* @param {float} My moment y
	* @param {float} Mz moment z
	* @param {bool} [local=false] if y z are local coordinates or not
	* @returns {float} stress at [x,z] caused by N,My,Mz*/
	computeStress: function(y,z,N,My,Mz,local) {
		var local = local || false;
		var a = this.a==null? this.computeA() : this.a;
		var iy = this.iy==null? this.computeIy() : this.iy;
		var iz = this.iz==null? this.computeIz() : this.iz;
		var dyz = this.dyz==null? this.computeDyz() : this.dyz;
		if (local) { var y=y; var z=z; }
		else {
			var y = y - (this.cy==null? this.computeCy() : this.cy);
			var z = z - (this.cz==null? this.computeCz() : this.cz);
		}
		return N/a + (My*iz+Mz*dyz)/(iy*iz-dyz*dyz)*z - (Mz*iy+My*dyz)/(iy*iz-dyz*dyz)*y;
	},

	/** Computes normal stress at certain polygon node
	* @param {Node} node polygon node
	* @param {float} N normal force
	* @param {float} My moment y
	* @param {float} Mz moment z
	* @returns {float} stress at node caused by N,My,Mz*/
	computeStressInNode: function(node,N,My,Mz) {
		return this.computeStress(node.y,node.z,N,My,Mz);
	},

	/** Computes neutral axis
	* @param {float} N normal force
	* @param {float} My moment y
	* @param {float} Mz moment z
	* @param {bool} [local=false] if returned equation is in local coordinates or not
	* @returns {[float,float,float]} [a,b,c] as ay+bz+c=0;*/
	computeNeutralAxis: function(N,My,Mz,local) {
		var local = local || false;
		var a = this.a==null? this.computeA() : this.a;
		var iy = this.iy==null? this.computeIy() : this.iy;
		var iz = this.iz==null? this.computeIz() : this.iz;
		var dyz = this.dyz==null? this.computeDyz() : this.dyz;
		var aa = -(Mz*iy+My*dyz)/(iy*iz-dyz*dyz);
		var bb = (My*iz+Mz*dyz)/(iy*iz-dyz*dyz);
		var cc = N/a;
		if (local) {
			var yc = this.cy==null? this.computeCy() : this.cy;
			var zc = this.cz==null? this.computeCz() : this.cz;
			cc += aa*yc+bb*zc;
		}
		return [aa,bb,cc];
	},

	/** Returns those nodes of receiver that forms the convex hull
	* @returns {[Nodes]} those nodes that forms convex hull (ret[0]==ret[ret.length-1])*/
	giveConvexHull: function() {
		if (this.nodes.length <= 4) { return this.nodes.slice(); }
		// choose two nodes (most lower-left and upper-right) that for sure forms a convex part
		var c1 = this.nodes[0];
		var c2 = this.nodes[0];
		var i1 = 0;
		var i2 = 0;
		for (var i=1; i<this.nodes.length; i++) {
			if (this.nodes[i].z < c1.z) { c1 = this.nodes[i]; i1 = i; continue; }
			if (this.nodes[i].z > c2.z) { c2 = this.nodes[i]; i2 = i; continue; }
			if (this.nodes[i].z == c1.z && this.nodes[i].y < c1.y) { c1 = this.nodes[i]; i1 = i; }
			if (this.nodes[i].z == c2.z && this.nodes[i].y > c2.y) { c2 = this.nodes[i]; i2 = i; }
		}

		n = this.nodes.slice(0,this.nodes.length-1)
		var nodes12 = i1<i2? n.slice(i1,i2) : n.slice(i1).concat(n.slice(0,i2));
		var nodes21 = i2<i1? n.slice(i2,i1) : n.slice(i2).concat(n.slice(0,i1));
		nodes12.push(c2); nodes21.push(c1);

		var v, c, s, cMin, iCur, cCur, vCur, l12 = nodes12.length, l21 = nodes21.length;
		var ret12 = []
		iCur = 0;
		if (l12 > 2) {
			do {
				ret12.push(nodes12[iCur]);
				cCur = nodes12[iCur]
				v = c2.sub(cCur)
				cMin = 1e20;
				for (var i=iCur+1; i<l12; i++) {
					vCur = nodes12[i].sub(cCur)
					c = vCur.dot(v)/(vCur.norm()*v.norm())
					if (c <= cMin) { if (vCur.cross(v).x >= 0.) { iCur = i; cMin=c; } }
				}
			} while (iCur<l12-1)
		} else { ret12 = [c1]; }

		var ret21 = [];
		iCur = 0;
		if (l21 > 2) {
			do {
				ret21.push(nodes21[iCur]);
				cCur = nodes21[iCur]
				v = c1.sub(cCur)
				cMin = 1e20;
				for (var i=iCur+1; i<l21; i++) {
					vCur = nodes21[i].sub(cCur)
					c = vCur.dot(v)/(vCur.norm()*v.norm())
					if (c <= cMin) { if (vCur.cross(v).x >= 0.) { iCur = i; cMin=c; } }
				}
			} while (iCur<l21-1)
		} else { ret21 = [c2]; }
		return ret12.concat(ret21).concat(c1)
	},

	/** Computes core (area, where applied force in x direction causes no tension)
	* @param {bool} [local=false] if computed with respect to this.cx center of mass or global yz coordinates
	* @returns {[Nodes]} array of nodes creating core polygon*/
	computeCore: function(local) {
		var core = [];
		var nodes = this.convexHull==null? this.giveConvexHull() : this.convexHull;
		var local = local || false;
		var a = this.a==null? this.computeA() : this.a;
		var iy = this.iy==null? this.computeIy() : this.iy;
		var iz = this.iz==null? this.computeIz() : this.iz;
		var dyz = this.dyz==null? this.computeDyz() : this.dyz;
		var yc = this.cy==null? this.computeCy() : this.cy;
		var zc = this.cz==null? this.computeCz() : this.cz;
		var y,z,n1,n2,y1,y2,z1,z2,cc;
		for (var i=0; i<nodes.length-1; i++) {
			n1 = nodes[i];
			n2 = nodes[i+1];
			y1 = n1.y - yc;
			y2 = n2.y - yc;
			z1 = n1.z - zc;
			z2 = n2.z - zc;
			//y = iz2*(z1-z2)/(y1*z2-y2*z1);
			//z = iy2*(y2-y1)/(y1*z2-y2*z1);
			cc = a*(y1*z2-y2*z1);
			y = (dyz*(y2-y1)+iz*(z1-z2))/cc;
			z = (dyz*(z1-z2)+iy*(y2-y1))/cc;
			core[i] = local? new Node(0.,y,z) : new Node(0.,y+yc,z+zc);
		}
		return core;
	},

	/** computes all values (a,sy,sz,cy,cz,iy,iz,dyz,i1,i2,beta,core) within one function call*/
	up: function() {
		this.a = this.computeA();
		this.sy = this.computeSy();
		this.sz = this.computeSz();
		this.cy = this.computeCy();
		this.cz = this.computeCz();
		this.iy = this.computeIy();
		this.iz = this.computeIz();
		this.dyz = this.computeDyz();
		var princ = this.computePrincipalValues();
		this.i1 = princ[0];
		this.i2 = princ[1];
		this.beta = princ[2];
		this.convexHull = this.giveConvexHull();
		this.core = this.computeCore();
	},
	/**#@-*/
}
/** Constructor, see {@link CrossSection} for input parameters description
* @returns {CrossSection} new CrossSection object*/
CrossSection.create = function(nodes,holes) {
	return new CrossSection(nodes,holes);
}
