/*!
*
*       JSMatrix : JavaScript matrix and vector implementation
*                      version 0.5 (2011-09-19)
*
*                 Copyright (C)  2011  Jan Stransky
*
*      Czech Technical University, Faculty of Civil Engineering,
*  Department of Structural Mechanics, 166 29 Prague, Czech Republic
*
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
* @fileOverview JavaScript vector and matrix implementation. For more information see <a href="http://mech.fsv.cvut.cz/~stransky/software/jsmatrix/">project homepage</a>. The inspiration of publication "Press, W. H., Teukolsky, S. A., Vetterling, W. T. and Flannery, B. P. 2007. The Numerical Recipes: The Art of Scientific Computing, Third Edition" is gratefully acknowledged.
<br /><br/>JSMatrix is a free software distributed under <a href='http://www.gnu.org/licenses/gpl.html'>GNU GPL license</a>.
* @author <a href="http://mech.fsv.cvut.cz/~stransky/">Jan Stránský</a>
* @version 0.5 (2011-09-19)
*/

// constants for easier manipulation
/**constant*/ var JSM_GAUSS       = 1;
/**constant*/ var JSM_GAUSSJORDAN = 2;
/**constant*/ var JSM_LU          = 3;
/**constant*/ var JSM_LDU         = 4;
/**constant*/ var JSM_CHOLESKY    = 5;
/**constant*/ var JSM_QR          = 6;
/**constant*/ var JSM_INV_IT      = 7;
/**constant*/ var JSM_SUBSPACE    = 8;

/**constant*/ var JSM_NOPIVOT   = 9;
/**constant*/ var JSM_PARTPIVOT = 10;
/**constant*/ var JSM_FULLPIVOT = 11;

/**constant ( = 0 )*/ var X = 0;
/**constant ( = 1 )*/ var Y = 1;
/**constant ( = 2 )*/ var Z = 2;
/**constant ( = 1e-8)*/ var JSM_TOL = 1e-8;

/**@ignore*/ var JSM_DEBUG = 0;
if (JSM_DEBUG) {
	/**@ignore*/
	assert = function(what,msg) {
		if (!what) { alert("JSM_DEBUG:\n"+(msg||"")); }
	}
} else {
	/**@ignore*/
	assert = function() {}
}





/** Vector implementation
* @class represents a vector (1D matrix) of real (floating point) numbers*/
Vector = function() {
	/** @property {int} [nItems=0] Number of items (elements) of receiver*/
	this.nItems = 0;
	/** @property {[floats]} [theArray=[]] array containing receiver's elements*/
	this.theArray = [];
}
Vector.prototype = {
	/**#@+
	* @function
	* @memberOf Vector#
	*/

	/** Zeros all elements of receiver
	* @example v = vec([1,2,3,4]);
	* v.zero(); // v = Vector([0,0,0,0])*/
	zero: function() {
		for (var r=0; r<this.nItems; r++) { this.theArray[r] = 0.; }
	},
	
	/** Returns size (numer of elements) of receiver
	* @returns {int} length (number of elements) of receiver
	* @example v = vec([1,5,8,2]); var s = v.size(); // s = 4*/
	size: function() {
		return this.nItems;
	},

	/** Testing vector equality
	* @param {Vector} vec vector to be compared with receiver
	* @returns {bool} true if this==vec, false otherwise
	* @example a = vec([1,2,3]);
	* b = vec([1,2,2.999999999]);
	* c = vec([1,2,1]);
	* t1 = a.isEqualTo(b); // t1 = true
	* t2 = a.isEqualTo(c); // t2 = false*/
	isEqualTo: function(vec) {
		if (!this.isSameSizeAs(vec)) { return false; }
		var v = vec.theArray;
		for (var r=0; r<this.nItems; r++) {
			if (Math.abs(this.theArray[r]-v[r]) > JSM_TOL) { return false; }
		}
		return true;
	},

	/** Returns one element of receiver ( ret = this[r] ) - numbering from 0
	* @param {int} r index of row (element) to be get (numbering from 0)
	* @returns {float} value of the r-th element
	* @example v = vec([4,7,2,4]); a = v.get(1); // a = 7*/
	get: function(r) {
		//assert(r < this.nItems,"Vector.get: r > this.nItems");
		return this.theArray[r];
	},

	/** Set one element of receiver ( this[r] = val ) - numbering from 0
	* @param {int} r index of row (element) to be set (numbering from 0)
	* @param {float} val value to be set
	* @example v = vec([4,7,2,4]); v.set(1,68); // v = Vector([4,68,2,4])*/
	set: function(r,val) {
		this.theArray[r] = val;
	},

	/** Increment one element of receiver ( this[r] += val ) - numbering from 0
	* @param {int} r index of row (element) to be incremented (numbering from 0)
	* @param {float} val value to be add
	* @example v = vec([4,7,2,4]); v.incr(1,4); // v = vec([4,11,2,4])*/
	incr: function(r,val) {
		if (val) { this.theArray[r] += val; }
	},

	/*
	/* * Returns one element of receiver ( ret = this[r] ) - numbering from 1
	* @param {int} r index of row (element) to be get (numbering from 1)
	* @returns {float} value of the r-th element
	* @example v = vec([4,7,2,4]); a = v.get1(1); // a = 4* /
	get1: function(r) {
		return this.theArray[r-1];
	},

	/* * Set one element of receiver ( this[r] = val ) - numbering from 1
	* @param {int} r index of row (element) to be set (numbering from 1)
	* @param {float} val value to be set
	* @example
	* v = vec([4,7,2,4]); v.set1(1,68); // v = vec([68,7,2,4])* /
	set1: function(r,val) {
		this.theArray[r-1] = val;
	},

	/* * Increment one element of receiver ( this[r] += val ) - numbering from 1
	* @param {int} r index of row (element) to be incremented (numbering from 1)
	* @param {float} val value to be add
	* @example v = vec([4,7,2,4]); v.incr1(1,4); // v = vec([8,7,2,4])* /
	incr1: function(r,val) {
		if (val) { this.theArray[r-1] += val; }
	},
	*/

	/** Testing vector's size equality
	* @param {Vector} vec vector to be tested
	* @returns {bool} true if vec and receiver has same size, false otherwise
	* @example a = vec([1,2,3,4]);
	* b = vec([5,6,7,8,9,10,11,12]);
	* c = vec([13,14,15,16]);
	* t1 = a.isSameSizeAs(b); // t1 = false
	* t2 = a.isSameSizeAs(c); // t1 = true*/
	isSameSizeAs: function(vec) {
		return (this.nItems==vec.nItems);
	},
	
	/** Testing if receiver can multiply given matrix ( this^T * mat )
	* @param {Matrix} mat matrix to be tested
	* @returns {bool} true if the multiplication is possible, false otherwise
	* @example v = vec([1,2,3]);
	* m1 = mat([[11,12,13],[21,22,23]]);
	* m2 = [[11,12],[21,22],[31,32]];
	* t1 = v.canMultiplyMat(m1); // t1 = true
	* t2 = v.canMultiplyMat(m2); // t2 = false*/
	canMultiplyMat: function(mat) {
		return this.nItems==mat.nCols;
	},

	/** Sets elements of receiver form given array
	* @param {[floats]} arry array containing new receiver elements
	* @example v.fromArray([3,6,5,2]); // v = Vector([3,6,5,2])*/
	fromArray: function(arry) {
		this.nItems = arry.length;
		this.theArray = arry.slice();
	},

	/** Returns elements of receiver as an array
	* @returns {[floats]} array containing receiver's elements
	* @example v =vec([3,4,5,6]); s = v.toArray(); // s = [3,4,5,6]*/
	toArray: function() {
		return this.theArray.slice();
	},

	/** Returns sum of receiver and vector ( ret = this + vec )
	* @param {Vector} vec vector to be added
	* @returns {Vector} sum of receiver and vec
	* @example v1 = vec([1,2,3]);
	* v2 = vec([6,1,5]);
	* v3 = v1.add(v2); // v3 = Vector([7,3,8])*/
	add: function(vec) {
		if (!this.isSameSizeAs(vec)) { return null; }
		var v = vec.theArray;
		var ret = [];
		for (var r=0; r<this.nItems; r++) { ret[r] = this.theArray[r] + v[r]; }
		return Vector.create(ret);
	},

	/** Add given vector to receiver ( this += vec )
	* @param {Vector} vec vector to be added
	* @example v1 = vec([1,2,3]);
	* v2 = vec([6,1,5]);
	* v1.iadd(v2); // v1 = Vector([7,3,8])*/
	iadd: function(vec) {
		if (!this.isSameSizeAs(vec)) { return null; }
		var v = vec.theArray;
		for (var r=0; r<this.nItems; r++) { this.theArray[r] += v[r]; }
	},

	/** Returns difference of receiver and vector ( ret = this - vec )
	* @param {Vector} vec vector to be substracted
	* @returns {Vector} difference of receiver and vec
	* @example v1 = vec([1,2,3]);
	* v2 = vec([6,1,5]);
	* v3 = v1.sub(v2); // v3 = Vector([-5,1,-2])*/
	sub: function(vec) {
		if (!this.isSameSizeAs(vec)) { return null; }
		var v = vec.theArray;
		var ret = [];
		for (var r=0; r<this.nItems; r++) { ret[r] = this.theArray[r] - v[r]; }
		return Vector.create(ret);
	},

	/** Substract given vector to receiver ( this -= vec )
	* @param {Vector} vec vector to be substracted
	* @example v1 = vec([1,2,3]);
	* v2 = vec([6,1,5]);
	* v1.isub(v2); // v1 = Vector([-5,1,-2])*/
	isub: function(vec) {
		if (!this.isSameSizeAs(vec)) { return null; }
		var v = vec.theArray;
		for (var r=0; r<this.nItems; r++) { this.theArray[r] -= v[r]; }
	},

	/** Dot product of receiver and given vector
	* @param {Vector} vec vector for dot product
	* @param {int} [n=0] dot prodact will be made from first n elements. If not specified, zero or negative, all elements are used
	* @returns {float} dot product of receiver and vec from first n elements
	* @example v1 = vec([1,2,3]);
	* v2 = vec([6,1,5]);
	* d = v1.dot(v2,2); // d = 8*/
	dot: function(vec,n) {
		var n = n || 0;
		if (n<=0 || n>this.nItems) { n = this.nItems; }
		if (n > vec.nItems) { return null; }
		var v = vec.theArray;
		var ret = 0.;
		for (var r=0; r<n; r++) { ret += this.theArray[r]*v[r]; }
		return ret;
	},

	/** Returns squared (Eucleidian) norm of receiver (ret = this.dot(this) )
	* @returns {float} squared Eucleidian norm of receiver
	* @example v = vec([4,2,4]);
	* n = v.squaredNorm(); // n = 36*/
	squaredNorm: function() {
		return this.dot(this);
	},

	/** Returns (Eucleidian) norm of receiver (ret = sqrt(this.dot(this)) )
	* @returns {float} Eucleidian norm of receiver
	* @example v = vec([4,2,4]);
	* n = v.norm(); // n = 6*/
	norm: function() {
		return Math.sqrt(this.dot(this));
	},

	/** Normlize receiver
	* @example v = vec([4,2,4]);
	* v.normalize(); // v = Vector([0.6666,0.5,0.66666])*/
	normalize: function() {
		var n = this.norm();
		this.imulf(1/n);
	},

	/** Returns normalized copy of receiver
	* @returns {Vector} normalized copy of receiver
	* @example v1 = vec([4,2,4]);
	* v2 = v1.normalized(); // v2 = Vector([0.6666,0.5,0.66666])*/
	normalized: function() {
		var ret = this.copy();
		ret.normalize();
		return ret;
	},

	/** Returns outer (dyadic, tensor, ...) product of receiver and given vector
	* @param {Vector} vec vector for multiplication
	* @returns {Matrix} outer product of receiver and vec
	* @example v1 = vec([1,2,3]);
	* v2 = vec([4,5,6]);
	* m = v1.outer(v2); // m = Matrix([[4,5,6],[8,10,12],[12,15,18]])*/
	outer: function(vec) {
		var ret = [];
		var v = vec.theArray;
		for (var r=0; r<this.nItems; r++) {
			ret[r] = [];
			for (var c=0; c<v.length; c++) { ret[r][c] = this.theArray[r] * v[c]; }
		}
		return Matrix.create(ret);
	},

	/** Returns vector (cross) product of receiver and given vector. Both receiver and given vector have to be of length 3.
	* @param {Vector} vec vector for multiplication
	* @returns {Vector} cross product of receiver and vec
	* @example v1 = vec([1,2,3]);
	* v2 = vec([4,5,6]);
	* v3 = v1.cross(v2); // v = Vector([-3,6,-3])*/
	cross: function(vec) {
		if (this.nItems!=3 || vec.nItems!=3) { return null; }
		var v = vec.theArray;
		return Vector.create([
			this.theArray[Y]*v[Z]-this.theArray[Z]*v[Y],
			this.theArray[Z]*v[X]-this.theArray[X]*v[Z],
			this.theArray[X]*v[Y]-this.theArray[Y]*v[X]]);
	},

	/** Returns negative of receiver ( ret = -this )
	* @returns {Vector} negative of receiver
	* @example v1 = vec([1,2,3]);
	* v2 = v1.negated(); // v2 = Vector([-1,-2,-3])*/
	negated: function() {
		var ret = this.copy();
		return ret.mulf(-1.);
	},

	/** Alias for negated(), see {@link Vector#negated}*/
	neg: function() { return this.negated(); },

	/** Negate receiver ( ret *= -1., ret = -ret )
	* @example v1 = vec([1,2,3]);
	* v1.negate(); // v1 = Vector([-1,-2,-3])*/
	negate: function() {
		for (var r=0; r<this.nItems; r++) { this.theArray[r] *= -1.; }
	},

	/** Alias for neagete(), see {@link Vector#negate}*/
	ineg: function() { this.negate(); },
	
	/** Returns receiver multiplied by float f ( ret = this * f )
	* @param {float} f float multiplier
	* @returns {Vector} copy of receiver multiplied by f
	* @example v1 = vec([1,2,3]);
	* v2 = v1.mulf(3); // v2 = Vector([3,6,9])*/
	mulf: function(f) {
		var ret = [];
		for (var r=0; r<this.nItems; r++) { ret[r] = f*this.theArray[r]; }
		return Vector.create(ret);
	},

	/** Multiply receiver by float f ( this *= f )
	* @param {float} f float multiplier
	* @example v = vec([1,2,3]);
	* v.imulf(3); // v = Vector([3,6,9])*/
	imulf: function(f) {
		for (var r=0; r<this.nItems; r++) { this.theArray[r] *= f; }
	},

	/** Returns product of receiver and given matrix ( ret = this^T * mat )
	* @param {Matrix} mat matrix to multiply
	* @returns {Vector} copy of receiver multiplied by mat
	* @example v1 = vec([1,2,3]);
	* m = mat([[11,12,13],[21,22,23],[31,32,33]])
	* v2 = v1.mulm(m); // v2 = Vector([146,152,158])*/
	mulm: function(mat) {
		if (!this.canMultiplyMat(mat)) { return null; }
		var m = mat.theArray;
		var ret = [];
		var temp;
		for (var c=0; c<mat.nCols; c++) {
			temp = 0.;
			for (var r=0; r<this.nItems; r++) {
				temp += this.theArray[r]*m[r][c];
			}
			ret[c] = temp;
		}
		return Vector.create(ret);
	},

	/** Returns product of receiver and given mltiplier (float or Matrix)
	* @param {Matrix|float} what matrix or float to multiply
	* @returns {Vector} copy of receiver multiplied by what
	* @example v1 = vec([1,2,3]);
	* v2 = v1.mul(mat([[11,12,13],[21,22,23],[31,32,33]])); // v2 = Vector([146,152,158])
	* v3 = v1.mul(3); // v3 = Vector([3,6,9])*/
	mul: function(what) {
		if (what instanceof Matrix) { return this.mulm(what); }
		if (typeof what == 'number') { return this.mulf(what); }
		return null;
	},

	/** Alias for mul(), see {@link Vector#mul}*/
	x: function(what) { return this.mul(what); },

	/** Returns subvector of receiver - numbering from 0
	* @param {[floats]|Vector} items array containing row indices of desired subvector - numbering from 0
	* @returns {Vector} desired subvector
	* @example v1 = vec([4,7,9,1,8,3,2,4,6,7,5,1,6,9]);
	* v2 = v1.getsv([2,3,6,8,9]); // v2 = Vector([9,1,2,6,7])*/
	getsv: function(items) {
		var items = items.theArray || items;
		var n = items.length;
		var ret = [];
		for (var r=0; r<n; r++) { ret[r] = this.theArray[items[r]]; }
		return Vector.create(ret);
	},

	/** Set subvector to defined positions of receiver
	* @param {[floats]|Vector} items array containing row indices to be set - numbering from 0
	* @param {Vector|[floats]} vec vector to be set
	* @example v = vec([4,7,9,1,8,3,2,4,6,7,5,1,6,9]);
	* v.setsv([2,3,6,8,9],vec([1.1,2.2,3.3,4.4,5.5])); // v = Vector([4,7,1.1,2.2,8,3,3.3,4,4.4,5.5,5,1,6,9])*/
	setsv: function(items,vec) {
		var v = vec.theArray || vec;
		var items = items.theArray || items;
		var n = items.length;
		if (v.length != n) { return; }
		for (var r=0; r<n; r++) { this.theArray[items[r]] = v[r]; }
	},

	/** Increment subvector to defined positions of receiver
	* @param {[floats]|Vector} items array containing row indices to be incremented - numbering from 0
	* @param {Vector|[floats]} vec vector to be added
	* @example v = vec([4,7,9,1,8,3,2,4,6,7,5,1,6,9]);
	* v.incrsv([2,3,6,8,9],vec([1.1,2.2,3.3,4.4,5.5])); // v = Vector([4,7,10.1,3.2,8,3,5.3,4,10.4,12.5,5,1,6,9])*/
	incrsv: function(items,vec) {
		var v = vec.theArray || vec;
		var items = items.theArray || items;
		var n = items.length;
		if (v.length != n) { return; }
		for (var r=0; r<n; r++) { this.theArray[items[r]] += v[r]; }
	},

	/** Decrement subvector to defined positions of receiver
	* @param {[floats]|Vector} items array containing row indices to be decremented - numbering from 0
	* @param {Vector|[floats]} vec vector to be substracted
	* @example v = vec([4,7,9,1,8,3,2,4,6,7,5,1,6,9]);
	* v.decrsv([2,3,6,8,9],vec([1.1,2.2,3.3,4.4,5.5])); // v = Vector([4,7,7.9,-1.2,8,3,-1.3,4,1.6,1.5,5,1,6,9])*/
	decrsv: function(items,vec) {
		var v = vec.theArray || vec;
		var items = items.theArray || items;
		var n = items.length;
		if (v.length != n) { return; }
		for (var r=0; r<n; r++) { this.theArray[items[r]] -= v[r]; }
	},

	/** Returns copy of receiver
	* @returns {Vector} copy of receiver
	* @example v1 = vec([1,2,3]);
	* v2 = v1;
	* v3 = v1.copy();
	* v1.set(1,6);
	* // v1; = Vector([1,6,3])
	* // v2; = Vector([1,6,3])
	* // v3; = Vector([1,2,3])*/
	copy: function() {
		var ret = new Vector();
		ret.fromArray(this.theArray.slice());
		return ret;
	},

	/** Returns string representation of receiver
	* @returns {string} string representation of receiver
	* @example v1 = vec([1,2,3]);
	* alert(v1); // alerts "Vector([ 1, 2, 3 ])"*/
	toString: function() {
		var nn = this.nItems;
		if (nn == 0) { return "Vector([])\n"; }
		var ret = "Vector([ "
		for (var r=0; r<nn-1; r++) { ret += this.theArray[r]+", "; }
		ret += this.theArray[nn-1]+" ])\n";
		return ret;
	},

	/** Swaps two items (items, elements..) of receiver - numbering from 0
	* @param {int} r1 index of first row to swap - numbering from 0
	* @param {int} r2 index of second row to swap - numbering from 0
	* @example v = vec([1,2,3,4,5,6,7]);
	* v.swapItems(1,4); // v = Vector([1,5,3,4,2,6,7])*/
	swapItems: function(r1,r2) {
		var row1 = this.theArray[r1];
		this.theArray[r1] = this.theArray[r2];
		this.theArray[r2] = row1;
	},

	/** Appends given float/vector to the end of receiver
	* @param {float|Vector|[Vectors]} items new row(s) to be appended
	* @example v = vec([1,2]);
	* v.appendItems(4); // v = Vector([1,2,4]);
	* v.appendItems(vec([7,6])); // v = Vector([1,2,4,7,6]);
	* v.appendItems([vec([2,1]),vec([33,44])]); // v = Vector([1,2,4,7,6,2,1,33,44])*/
	appendItems: function(items) {
		if (items instanceof Vector) {
			for (var i=0; i<items.nItems; i++) {
				this.theArray[this.nItems] = items.theArray[i];
				this.nItems++;
			}
			return;
		}
		if (items instanceof Array) {
			for (var row=0; row<items.length; row++) {
				this.appendItems(items[row]);
			}
			return;
		}
		if (typeof items == 'number') {
			this.theArray[this.nItems] = items;
			this.nItems++;
			return;
		}
	},

	/** Permute items of receiver according to given indices
	* @ param {[ints]|Vector} indx indices of permutation
	* @ param {bool} [backward=false] if false, receiver is permutated to given indices (forward). If true, from given indices (backward)
	* @ example v = vec([7,9,6]);
	* v.permuteItems([1,2,0]); // v = Vector([9,6,7])
	* v.permuteItems([1,2,0],true); // v = Vector([7,9,6])*/
	permuteItems: function(indx,backward) {
		var indx = indx.theArray || indx;
		if (indx.length != this.nItems) { return; }
		var backward = backward || false;
		var newArry = [];
		if (backward) {
			for (var i=0; i<this.nItems; i++) {
				newArry[i] = this.theArray[indx.indexOf(i)];
			}
			this.fromArray(newArry);
			return;
		}
		for (var i=0; i<this.nItems; i++) {
			newArry[i] = this.theArray[indx[i]];
		}
		this.fromArray(newArry);
	},

	/** Returns permutated copy of receiver according to given indices
	* @ param {[ints]|Vector} indx indices of permutation
	* @ param {bool} [backward=false] if false, receiver is permutated to given indices (forward). If true, from given indices (backward)
	* @returns {Vector} permutated copy of receiver
	* @ example v1 = vec([7,9,6]);
	* v2 = v1.itemPermutation([1,2,0]); // v2 = Vector([9,6,7])
	* v3 = v1.itemPermutation([1,2,0],true); // v3 = Vector([6,7,9])*/
	itemPermutation: function(indx,backward) {
		var indx = indx.theArray || indx;
		if (indx.length != this.nItems) { return null; }
		var ret = this.copy();
		ret.permuteItems(indx,backward);
		return ret;
	},

	/** Resize receiver according to given size (delete extra elements or add zero elements)
	* @param {int} nItems new number of items
	* @example v1 = vec([4,7,9,1,7,3])
	* v2 = vec([4,6]);
	* v1.resize(4); // v1 = Vector([4,7,9,1])
	* v2.resize(4); // v2 = Vector([4,6,0,0])*/
	resize: function(nItems) {
		if (nItems==this.nItems) { return; }
		if (nItems > this.nItems) {
			for (var r=this.nItems; r<nItems; r++) { this.theArray[r] = 0.; }
			this.nItems = nItems;
			return;
		}
		if (nItems < this.nItems) {
			this.theArray = this.theArray.slice(0,nItems);
			this.nItems = nItems;
		}
	},
	/** Returns resized copy of receiver according to given size (delete extra elements or add zero elements)
	* @param {int} nItems new number of items
	* @returns {Vector} resized copy of receiver
	* @example v1 = vec([4,7,9,1]);
	* v2 = v1.resized(2); // v2 = Vector([4,7])
	* v3 = v1.resized(6); // v3 = Vector([4,7,9,1,0,0])*/
	resized: function(nItems) {
		var ret = this.copy();
		ret.resize(nItems);
		return ret;
	},

	/** Returns matrix with receiver's elements on its diagonal
	* @returns {Matrix} diagonal matrix with receiver's elements on its diagonal
	* @example v = vec([1,2,3]);
	* m = v.toDiagonalMatrix(); // m = Matrix([[1,0,0],[0,2,0],[0,0,3]])*/
	toDiagonalMatrix: function() {
		var ret = Matrix.Zeros(this.nItems,this.nItems);
		for (var r=0; r<this.nItems; r++) { ret.theArray[r][r] = this.theArray[r]; }
		return ret;
	},

	/** Alias for toDiagonalMatrix(), see {@link Vector#toDiagonalMatrix}*/
	diag: function() { return this.toDiagonalMatrix(); },

	/**#@-*/
}

/** Constructs new Vector from given array
* @param {[floats]} [arry=[]] array containing elements of new vector
* @returns {Vector} new Vector object 
* @example v = Vector.create([1,2,3,4]) // v = Vector([1,2,3,4])*/
Vector.create = function(arry) {
	var a = (arry==null)? []:arry.slice();
	var ret = new Vector();
	ret.fromArray(a);
	return ret;
}

/** Creates unit vector in x direction
* @returns {Vector} unit x vector
* @example v = UnitX // v = Vector([1,0,0])*/
Vector.UnitX = Vector.create([1,0,0]);

/** Creates unit vector in y direction
* @returns {Vector} unit y vector
* @example v = UnitX // v = Vector([0,1,0])*/
Vector.UnitY = Vector.create([0,1,0]);

/** Creates unit vector in z direction
* @returns {Vector} unit z vector
* @example v = UnitX // v = Vector([0,0,1])*/
Vector.UnitZ = Vector.create([0,0,1]);

/** Creates a vector full of zeros
* @param {int} [nItems=0] number of elements
* @returns {Vector} new vector full of zeros
* @example v = Vector.Zeros(4) // v = Vector([0,0,0,0])*/
Vector.Zeros = function(nItems) {
	var nItems = nItems || 0;
	var arry = [];
	for (var i=0; i<nItems; i++) { arry.push(0.); }
	return Vector.create(arry);
}

/** Creates a vector full of ones
* @param {int} [nItems=0] number of elements
* @returns {Vector} new vector full of zeros
* @example v = Vector.Ones(6) // v = Vector([1,1,1,1,1,1])*/
Vector.Ones = function(nItems) {
	var nItems = nItems || 0;
	var arry = [];
	for (var i=0; i<nItems; i++) { arry.push(1.); }
	return Vector.create(arry);
}





/** Matrix implementation
* @class represents a 2D matrix of real (floating point) numbers
* @property {int} [nRows=0] Number of rows of receiver
* @property {int} [nCols=0] Number of columns of receiver
* @property {[floats]} [theArray=[]] Array (2D) containing receiver's elements*/
Matrix = function(nRows,nCols) {
	this.nRows = nRows || 1;
	this.nCols = nCols || 0;
	this.theArray = [[]];
}
Matrix.prototype = {
	/**#@+
	* @function
	* @memberOf Matrix#
	*/

	/** Zeros all elements of receiver
	* @example m = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* m.zero(); // m = Matrix([[0,0,0],[0,0,0],[0,0,0]])*/
	zero: function() {
		for (var r=0; r<this.nRows; r++) {
			this.theArray[r] = []
			for (var c=0; c<this.nCols; c++) { this.theArray[r][c] = 0.; }
		}
	},

	/** Returns size of receiver as [nRows,nCols]
	* @returns {[int,int]} array as [number of rows,number of columns]
	* @example m = mat([[1,2,3],[4,5,6]]);
	* s = m.size(); // s = [2,3]*/
	size: function() {
		return [this.nRows,this.nCols];
	},

	/** Testing matrix equality
	* @param {Matrix} mat matrix to be compared with receiver
	* @returns {bool} true if this==mat, false otherwise
	* @example a = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* b = mat([[1,2,2.999999999],[4.0000000000002,5,6],[7,8,8.9999999999]]);
	* c = mat([[1,2,3],[4,5,6],[7,8,9],[10,11,12]]);
	* t1 = a.isEqualTo(b); // t1 = true
	* t2 = a.isEqualTo(c); // t2 = false*/
	isEqualTo: function(mat) {
		if (!this.isSameSizeAs(mat)) { return false; }
		var m = mat.theArray;
		for (var r=0; r<this.nRows; r++) {
			for (var c=0; c<this.nCols; c++) {
				if (Math.abs(this.theArray[r][c] - m[r][c]) > JSM_TOL) { return false; }
			}
		}
		return true;
	},

	/** Returns one element of receiver ( ret = this[r,c] ) - numbering from 0
	* @param {int} r index of row to be get (numbering from 0)
	* @param {int} c index of column to be get (numbering from 0)
	* @returns {float} value of the element at r-th row and c-th column
	* @example m = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* g = m.get(1,2); // g = 23*/
	get: function(r,c) {
		return this.theArray[r][c];
	},

	/** Set one element of receiver ( this[r,c] = val ) - numbering from 0
	* @param {int} r index of row to be set (numbering from 0)
	* @param {int} c index of column to be set (numbering from 0)
	* @param {float} val value to be set
	* @example m = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m.set(1,2,3.45); // m = Matrix([[11,12,13],[21,22,3.45],[31,32,33]])
	*/
	set: function(r,c,val) {
		this.theArray[r][c] = val;
	},

	/** Increment one element of receiver ( this[r,c] += val ) - numbering from 0
	* @param {int} r index of row to be incremented (numbering from 0)
	* @param {int} c index of column to be incremented (numbering from 0)
	* @param {float} val value to be add
	* @example m = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m.incr(1,2,3.45); // m = Matrix([[11,12,13],[21,22,26.45],[31,32,33]])
	*/
	incr: function(r,c,val) {
		if (val) { this.theArray[r][c] += val; }
	},

	/** Testing matrix size equality
	* @param {Matrix} mat matrix to be tested
	* @returns {bool} true if mat and receiver has same size, false otherwise
	* @example a = mat([[1,2,3],[4,5,6]]);
	* b = mat([[5,6,7],[8,9,10],[11,12,13]]);
	* c = mat([[14,15,16],[17,18,19]]);
	* t1 = a.isSameSizeAs(b); // t1 = false
	* t2 = a.isSameSizeAs(c); // t1 = true*/
	isSameSizeAs: function(mat) {
		return (this.nRows==mat.nRows && this.nCols==mat.nCols);
	},

	/** Testing if receiver can multiply given matrix ( this * mat )
	* @param {Matrix} mat matrix to be tested
	* @returns {bool} true if the multiplication is possible, false otherwise
	* @example m1 = mat([[1,2,3],[4,5,6]]);
	* m2 = mat([[7,8,9],[10,11,12]]);
	* m3 = mat([[11,12],[21,22],[31,32]]);
	* t1 = m1.canMultiplyMat(m2); // t1 = false
	* t2 = m1.canMultiplyMat(m3); // t2 = true*/
	canMultiplyMat: function(mat) {
		return (this.nCols==mat.nRows);
	},

	/** Testing if receiver can multiply given vector ( this * vec )
	* @param {Vector} vec vector to be tested
	* @returns {bool} true if the multiplication is possible, false otherwise
	* @example m = mat([[1,2,3],[4,5,6]]);
	* v1 = vec([1,2,3]);
	* v2 = vec([4,5]);
	* t1 = m.canMultiplyVec(v1); // t1 = true
	* t2 = m.canMultiplyVec(v2); // t2 = false*/
	canMultiplyVec: function(vec) {
		return (this.nCols==vec.nItems);
	},

	/** Testing if receiver is square
	* @returns {bool} true if receiver is square matrix, false otherwise
	* @example m1 = mat([[1,2,3],[4,5,6]]);
	* m2 = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* t1 = m1.isSquare(); // t1 = false
	* t2 = m2.isSquare(); // t2 = true*/
	isSquare: function() {
		return (this.nRows==this.nCols);
	},

	/** Testing if receiver is symmetric matrix
	* @returns {bool} true if receiver is symmetric, false otherwise
	* @example m1 = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m2 = mat([[11,12,13],[12,22,23],[13,23,33]]);
	* t1 = m1.isSymmetric(); // t1 = false
	* t2 = m2.isSymmetric(); // t2 = true*/
	isSymmetric: function() {
		if (!this.isSquare()) { return false; }
		for (var r=0; r<this.nRows; r++) {
			for (var c=r+1; c<this.nCols; c++) {
				if (Math.abs(this.theArray[r][c] - this.theArray[c][r]) > JSM_TOL) { return false; }
			}
		}
		return true;
	},

	/** Testing if receiver is lower triangular matrix
	* @returns {bool} true if receiver is lower triangular, false otherwise
	* @example m1 = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* m2 = mat([[1,0,0],[4,5,0],[7,8,9]]);
	* t1 = m1.isLowerTriangular(); // t1 = false
	* t2 = m2.isLowerTriangular(); // t2 = true*/
	isLowerTriangular: function() {
		for (var r=0; r<this.nRows; r++) {
			for (var c=r+1; c<this.nCols; c++) {
				if (Math.abs(this.theArray[r][c]) > JSM_TOL) { return false; }
			}
		}
		return true;
	},

	/** Testing if receiver is upper triangular matrix
	* @returns {bool} true if receiver is upper triangular, false otherwise
	* @example m1 = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* m2 = mat([[1,2,3],[0,5,6],[0,0,9]]);
	* t1 = m1.isUpperTriangular(); // t1 = false
	* t2 = m2.isUpperTriangular(); // t2 = true*/
	isUpperTriangular: function() {
		for (var r=0; r<this.nRows; r++) {
			for (var c=0; c<r; c++) {
				if (Math.abs(this.theArray[r][c]) > JSM_TOL) { return false; }
			}
		}
		return true;
	},

	/** Sets elements of receiver form given array
	* @param {[floats](2D)} arry array containing new receiver elements
	* @example m.fromArray([[1,2,3],[4,5,6],[7,8,9]]); // m = Matrix([[1,2,3],[4,5,6],[7,8,9]])*/
	fromArray: function(arry) {
		this.nRows = arry.length;
		if (this.nRows) { this.nCols = arry[0].length; }
		else { this.nCols = 0; }
		this.theArray = []
		for (var r=0; r<this.nRows; r++) {
			this.theArray[r] = arry[r].slice();
		}
	},

	/** Returns elements of receiver as an array
	* @returns {[floats](2D)} array containing receiver's elements
	* @example m =mat([[11,12],[21,22]]);
	* a = m.toArray(); // a = [[11,12],[21,22]]*/
	toArray: function() {
		var ret = []
		for (var r=0; r<this.nRows; r++) { ret[r] = this.theArray[r].slice(); }
		return ret;
	},

	/** Returns sum of receiver and matrix ( ret = this + mat )
	* @param {Matrix} mat vector to be added
	* @returns {Matrix} sum of receiver and matrix
	* @example m1 = mat([[1,2],[3,4]]);
	* m2 = mat([[2,5],[3,2]]);
	* m3 = m1.add(m2); // m3 = Matrix([[3,7],[6,6]])*/
	add: function(mat) {
		if (!this.isSameSizeAs(mat)) { return null; }
		var m = mat.theArray;
		var ret = [];
		for (var r=0; r<this.nRows; r++) {
			ret[r] = [];
			for (var c=0; c<this.nCols; c++) { ret[r][c] = this.theArray[r][c] + m[r][c]; }
		}
		return Matrix.create(ret);
	},

	/** Add given matrix to receiver ( this += mat )
	* @param {Matrix} mat vector to be added
	* @example m1 = mat([[1,2],[3,4]]);
	* m2 = mat([[2,5],[3,2]]);
	* m1.iadd(m2); // m1 = Matrix([[3,7],[6,6]])*/
	iadd: function(mat) {
		if (!this.isSameSizeAs(mat)) { return null; }
		var m = mat.theArray;
		for (var r=0; r<this.nRows; r++) {
			for (var c=0; c<this.nCols; c++) { this.theArray[r][c] += m[r][c]; }
		}
	},

	/** Returns difference of receiver and matrix ( ret = this - mat )
	* @param {Matrix} mat vector to be added
	* @returns {Matrix} sum of receiver and matrix
	* @example m1 = mat([[1,2],[3,4]]);
	* m2 = mat([[2,5],[3,2]]);
	* m3 = m1.sub(m2); // m3 = Matrix([[-1,-3],[0,2]])*/
	sub: function(mat) {
		if (!this.isSameSizeAs(mat)) { return null; }
		var m = mat.theArray;
		var ret = [];
		for (var r=0; r<this.nRows; r++) {
			ret[r] = [];
			for (var c=0; c<this.nCols; c++) { ret[r][c] = this.theArray[r][c] - m[r][c]; }
		}
		return Matrix.create(ret);
	},

	/** Substract given matrix to receiver ( this -= mat )
	* @param {Matrix} mat vector to be added
	* @example m1 = mat([[1,2],[3,4]]);
	* m2 = mat([[2,5],[3,2]]);
	* m1.isub(m2); // m1 = Matrix([[-1,-3],[0,2]])*/
	isub: function(mat) {
		if (!this.isSameSizeAs(mat)) { return null; }
		var m = mat.theArray;
		for (var r=0; r<this.nRows; r++) {
			for (var c=0; c<this.nCols; c++) { this.theArray[r][c] -= m[r][c]; }
		}
	},

	/** Returns negative of receiver ( ret = -this )
	* @returns {Matrix} negative of receiver
	* @example m1 = mat([[1,2,3],[4,5,6]]);
	* m2 = m1.negated(); // m2 = Matrix([[-1,-2,-3],[-4,-5,-6]])*/
	negated: function() {
		var ret = this.copy()
		return ret.mulf(-1.);
	},

	/** Alias for neageted(), see {@link Matrix#negated}*/
	neg: function() { return this.negated(); },

	/** Negate receiver ( ret *= -1., ret = -ret )
	* @example m1 = mat([[1,2,3],[4,5,6]]);
	* m1.negate(); // m1 = Matrix([[-1,-2,-3],[-4,-5,-6]])*/
	negate: function() {
		for (var r=0; r<this.nRows; r++) {
			for (var c=0; c<this.nCols; c++) { this.theArray[r][c] *= -1.; }
		}
	},

	/** Alias for neagete(), see {@link Matrix#negate}*/
	ineg: function() { this.negate(); },

	/** Returns one row of receiver - numbering from 0
	* @param {int} r index of row to return - numbering from 0
	* @returns {Vector} r-th row of receiver as vector
	* @example m = m([[1,2,3],[4,5,6],[7,8,9]]);
	* v = mat.getRow(1); // v = Vector([4,5,6])*/
	getRow: function(r) {
		return Vector.create(this.theArray[r].slice())
	},

	/** Returns one column of receiver - numbering from 0
	* @param {int} c index of column to return - numbering from 0
	* @returns {Vector} c-th column of receiver as vector
	* @example m = m([[1,2,3],[4,5,6],[7,8,9]]);
	* v = mat.getCol(1); // v = Vector([2,5,8])*/
	getCol: function(c) {
		var ret = []
		for (var r=0; r<this.nRows; r++) { ret[r] = this.theArray[r][c]; }
		return Vector.create(ret);
	},

	/** Sets one row of receiver - numbering from 0
	* @param {int} r index of row to set - numbering from 0
	* @param {Vector} vec vector to be set as new r-th row
	* @example mat([[1,2,3],[4,5,6],[7,8,9]]);
	* m.setRow(1,vec([11,12,13])); // m = Matrix([[1,2,3],[11,12,13],[7,8,9]])*/
	setRow: function(r,vec) {
		if (this.nCols != vec.nItems) { return; }
		var v = vec.theArray;
		this.theArray[r] = v.slice();
	},

	/** Sets one column of receiver - numbering from 0
	* @param {int} c index of column to set - numbering from 0
	* @param {Vector} vec vector to be set as new r-th row
	* @example m = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* m.setCol(1,vec([11,12,13])); // m = Matrix([[1,11,3],[4,12,6],[7,13,9]])*/
	setCol: function(c,vec) {
		if (this.nRows != vec.nItems) { return; }
		var v = vec.theArray;
		for (var r=0; r<this.nRows; r++) {
			this.theArray[r][c] = v[r];
		}
	},

	/** Swaps two rows of receiver - numbering from 0
	* @param {int} r1 index of first row to swap - numbering from 0
	* @param {int} r2 index of second row to swap - numbering from 0
	* @example m = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m.swapRows(0,2); // m = Matrix([[31,32,33],[21,22,23],[11,12,13]])*/
	swapRows: function(r1,r2) {
		var row1 = this.theArray[r1];
		this.theArray[r1] = this.theArray[r2];
		this.theArray[r2] = row1;
	},

	/** Swaps two columns of receiver - numbering from 0
	* @param {int} c1 index of first row to swap - numbering from 0
	* @param {int} c2 index of second row to swap - numbering from 0
	* @example m = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m.swapCols(0,2); // m = Matrix([[13,12,11],[23,22,21],[33,32,31]])*/
	swapCols: function(c1,c2) {
		var temp
		for (var r=0; r<this.nRows; r++) {
			temp = this.theArray[r][c1];
			this.theArray[r][c1] = this.theArray[r][c2];
			this.theArray[r][c2] = temp;
		}
	},

	/** Permute rows of receiver according to given indices
	* @ param {[ints]|Vector} indx indices of row permutation
	* @ param {bool} [backward=false] if false, receiver's rows is permutated to given indices (forward). If true, from given indices (backward)
	* @example m = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m.permuteRows([1,2,0]); // m = Matrix([[21,22,23],[31,32,33],[11,12,13]])
	* m.permuteRows([1,2,0],true); // m = mat([[11,12,13],[21,22,23],[31,32,33]])*/
	permuteRows: function(indx,backward) {
		var indx = indx.theArray || indx;
		if (indx.length != this.nRows) { return null; }
		var backward = backward || false;
		var newArry = [];
		if (backward) {
			for (var i=0; i<this.nRows; i++) {
				newArry[i] = this.theArray[indx.indexOf(i)].slice();
			}
			this.fromArray(newArry);
			return;
		}
		for (var i=0; i<this.nRows; i++) {
			newArry[i] = this.theArray[indx[i]].slice();
		}
		this.fromArray(newArry);
	},

	/** Returns copy of receiver with rows permutated according to given indices
	* @ param {[ints]|Vector} indx indices of permutation
	* @ param {bool} [backward=false] if false, receiver is permutated to given indices (forward). If true, from given indices (backward)
	* @returns {Matrix} copy of receiver with permutated rows
	* @example m1 = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m2 = m1.permuteRows([1,2,0]); // m2 = Matrix([[21,22,23],[31,32,33],[11,12,13]])
	* m3 = m1.permuteRows([1,2,0],true); // m3 = mat([[31,32,33],[11,12,13],[21,22,23]])*/
	rowPermutation: function(indx,backward) {
		var indx = indx.theArray || indx;
		var backward = backward || false;
		if (indx.length != this.nRows) { return null; }
		var ret = this.copy();
		ret.permuteRows(indx,backward);
		return ret;
	},

	/** Returns receiver multiplied by float f ( ret = this * f )
	* @param {float} f float multiplier
	* @returns {Matrix} copy of receiver multiplied by f
	* @example m1 = mat([[1,2,3],[4,5,6]]);
	* m2 = m1.mulf(3); // m2 = Matrix([[3,6,9],[12,15,18]])*/
	mulf: function(f) {
		var ret = this.copy()
		ret.imulf(f)
		return ret;
	},

	/** Multiply receiver by float f ( this *= f )
	* @param {float} f float multiplier
	* @example m = mat([[1,2,3],[4,5,6]]);
	* m.imulf(3); // m = Matrix([[3,6,9],[12,15,18]])*/
	imulf: function(f) {
		for (var r=0; r<this.nRows; r++) {
			for (var c=0; c<this.nCols; c++) { this.theArray[r][c] *= f; }
		}
	},

	/** Returns product of receiver and given vector ( ret = this * vec )
	* @param {Vector} vec vector to multiply
	* @returns {Vector} copy of receiver multiplied by vec
	* @example m = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* v1 = vec([1,2,3]);
	* v2 = m.mulv(v1); // v2 = Vector([74,134,194])*/
	mulv: function(vec) {
		if (!this.canMultiplyVec(vec)) { return null; }
		var v = vec.theArray;
		var ret = [];
		var temp;
		for (var r=0; r<this.nRows; r++) {
			temp = 0.
			for (var c=0; c<this.nCols; c++) {
				temp += this.theArray[r][c]*v[c]
			}
			ret[r] = temp;
		}
		return Vector.create(ret);
	},

	/** Returns product of receiver and given matrix ( ret = this * mat )
	* @param {Matrix} mat matrix to multiply
	* @returns {Matrix} copy of receiver multiplied by mat
	* @example m1 = mat([[11,12],[21,22]]);
	* m2 = mat([[1,2],[3,4]]);
	* m3 = m1.mulm(m2); // m3 = Matrix([[47,70],[87,130]])*/
	mulm: function(mat) {
		if (!this.canMultiplyMat(mat)) { return null; }
		var m = mat.theArray;
		var ret = [];
		var temp;
		for (var r=0; r<this.nRows; r++) {
			ret[r] = [];
			for (var c=0; c<mat.nCols; c++) {
				temp = 0.;
				for (var k=0; k<this.nCols; k++) {
					temp += this.theArray[r][k]*m[k][c]
				}
				ret[r][c] = temp;
			}
		}
		return Matrix.create(ret);
	},

	/** Returns product of receiver and given mltiplier (float or Matrix)
	* @param {Matrix|Vector|float} what matrix or vector or float to multiply
	* @returns {Matrix|Vector} copy of receiver multiplied by what
	* @example m1 = mat([[1,2,3],[4,5,6]]);
	* m2 = m1.mulf(3); // m2 = Matrix([[3,6,9],[12,15,18]])
	*
	* m3 = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* v1 = vec([1,2,3]);
	* v2 = m3.mulv(v1) // v2 = Vector([74,134,194]);
	*
	* m4 = mat([[11,12],[21,22]]);
	* m5 = mat([[1,2],[3,4]]);
	* m6 = m4.mulm(m5); // m6 = Matrix([[47,70],[87,130]])*/
	mul: function(what) {
		if (what instanceof Vector) { return this.mulv(what); }
		if (what instanceof Matrix) { return this.mulm(what); }
		if (typeof what == 'number') { return this.mulf(what); }
		return null;
	},

	/** Alias for mul(), see {@link Matrix#mul}*/
	x: function(what) { return this.mul(what); },

	/** Returns transposition of receiver ( ret = this^T )
	* @returns {Matrix} transposed copy of receiver
	* @example m1 = mat([[1,2,3],[4,5,6]]);
	* m2 = m1.transposed(); // m2 = Matrix([[1,4],[2,5],[3,6]])*/
	transposed: function() {
		var ret = [];
		for (var r=0; r<this.nCols; r++) {
			ret[r] = [];
			for (var c=0; c<this.nRows; c++) {
				ret[r][c] = this.theArray[c][r];
			}
		}
		return Matrix.create(ret);
	},

	/** Alias for transposed(), see {@link Matrix#transposed}*/
	T: function() { return this.transposed(); },

	/** Returns submatrix of receiver - numbering from 0
	* @param {[floats]|Vector} rows array containing rowa indices of desired submatrix - numbering from 0
	* @param {[floats]|Vector} cols array containing columns indices of desired submatrix - numbering from 0
	* @returns {Matrix} desired submatrix
	* @example m1 = mat([[11,12,13,14],[21,22,23,24],[31,32,33,34],[41,42,43,44]]);
	* m2 = m1.getsm([1,2],[0,2]); // m2 = Matrix([[21,23],[31,33]])*/
	getsm: function(rows,cols) {
		var nr = rows.nItems || rows.length
		var nc = cols.nItems || cols.length
		var rows = rows.theArray || rows;
		var cols = cols.theArray || cols;
		var ret = [];
		for (var r=0; r<nr; r++) {
			ret[r] = [];
			for (var c=0; c<nc; c++) { ret[r][c] = this.theArray[rows[r]][cols[c]]; }
		}
		return Matrix.create(ret);
	},

	/** Set submatrix to defined positions of receiver
	* @param {[floats]|Vector} rows array containing rows indices to be set - numbering from 0
	* @param {[floats]|Vector} cols array containing columns indices to be set - numbering from 0
	* @param {Matrix} mat matrix to be set on desired positions
	* @example m = mat([[11,12,13,14],[21,22,23,24],[31,32,33,34],[41,42,43,44]]);
	* m.setsm([1,2],[0,2],mat([[66,77],[88,99]]));
	* // m = Matrix([[11,12,13,14],[66,22,77,24],[88,32,99,34],[41,42,43,44]])*/
	setsm: function(rows,cols,mat) {
		var rows = rows.theArray || rows;
		var cols = cols.theArray || cols;
		var nr = rows.length;
		var nc = cols.length;
		if (mat.nRows != nr || mat.nCols != nc) { return; }
		var m = mat.theArray;
		for (var r=0; r<nr; r++) {
			for (var c=0; c<nc; c++) { this.theArray[rows[r]][cols[c]] = m[r][c]; }
		}
	},

	/** Increment submatrix at defined positions of receiver
	* @param {[floats]|Vector} rows array containing rows indices to be incrementes - numbering from 0
	* @param {[floats]|Vector} cols array containing columns indices to be incremented - numbering from 0
	* @param {Matrix} mat matrix to be incremented on desired positions
	* @example m = mat([[11,12,13,14],[21,22,23,24],[31,32,33,34],[41,42,43,44]]);
	* m.incrsm([1,2],[0,2],mat([[66,77],[88,99]]));
	* // m = Matrix([11,12,13,14],[87,22,110,24],[119,32,132,34],[41,42,43,44]])*/
	incrsm: function(rows,cols,mat) {
		var rows = rows.theArray || rows;
		var cols = cols.theArray || cols;
		var nr = rows.length;
		var nc = cols.length;
		if (mat.nRows != nr || mat.nCols != nc) { return; }
		var m = mat.theArray;
		for (var r=0; r<nr; r++) {
			for (var c=0; c<nc; c++) { this.theArray[rows[r]][cols[c]] += m[r][c]; }
		}
	},

	/** Decrement submatrix at defined positions of receiver
	* @param {[floats]|Vector} rows array containing rows indices to be decrementes - numbering from 0
	* @param {[floats]|Vector} cols array containing columns indices to be decremented - numbering from 0
	* @param {Matrix} mat matrix to be decremented on desired positions
	* @example m = mat([[11,12,13,14],[21,22,23,24],[31,32,33,34],[41,42,43,44]]);
	* m.decrsm([1,2],[0,2],mat([[66,77],[88,99]]));
	* // m = Matrix([11,12,13,14],[-45,22,-54,24],[-57,32,-66,34],[41,42,43,44]])*/
	decrsm: function(rows,cols,mat) {
		var rows = rows.theArray || rows;
		var cols = cols.theArray || cols;
		var nr = rows.length;
		var nc = cols.length;
		if (mat.nRows != nr || mat.nCols != nc) { return; }
		var m = mat.theArray;
		for (var r=0; r<nr; r++) {
			for (var c=0; c<nc; c++) { this.theArray[rows[r]][cols[c]] -= m[r][c]; }
		}
	},

	/** Returns copy of receiver
	* @returns {Matrix} copy of receiver
	* @example m1 = mat([[11,12],[21,22]]);
	* m2 = m1;
	* m3 = m1.copy();
	* m1.set(1,0,6);
	* // m1 = Matrix([[11,12],[6,22]])
	* // m2 = Matrix([[11,12],[6,22]])
	* // m3 = Matrix([[11,12],[21,22]])*/
	copy: function() {
		var ret = []
		for (var r=0; r<this.nRows; r++) { ret[r] = this.theArray[r].slice(); }
		return Matrix.create(ret);
	},

	/** Returns vector containing receiver's diagonal elements
	* @returns {Vector} vector containing receiver's diagonal elements
	* @example m = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* v = m.diag(); // v = Vector([1,5,9])*/
	diagonalToVector: function() {
		if (!this.isSquare()) { return null; }
		var ret = [];
		for (var r=0; r<this.nRows; r++) { ret[r] = this.theArray[r][r]; }
		return Vector.create(ret);
	},

	/** Alias for diagonalToVector(), see {@link Matrix#diagonalToVector}*/
	diag: function() { return this.diagonalToVector(); },

	/** Returns string representation of receiver
	* @returns {string} string representation of receiver
	* @example m = mat([[1,2,3],[4,5,6],[7,8,9]]);
	* alert(m); // alerts "Matrix([[1,2,3],[4,5,6],[7,8,9]])"*/
	toString: function() {
		var nr = this.nRows;
		var nc = this.nCols;
		var ret = "Matrix([[ "
		if (nr == 0) { return ret+"])"; }
		if (nc == 0) {
			for (var r=0; r<nr-1; r++) {
				ret += " [],";
			}
			return ret + " ])";
		}
		for (var r=0; r<nr-1; r++) {
			for (var c=0; c<nc-1; c++) { ret += this.theArray[r][c]+", "; }
			ret += this.theArray[r][nc-1]+" ], [ ";
		}
		for (var c=0; c<nc-1; c++) { ret += this.theArray[nr-1][c]+", "; }
		ret += this.theArray[nr-1][nc-1]+" ]])"
		return ret;
	},

	/** Resize receiver according to given size (delete extra elements or add zero elements)
	* @param {int} nRows new number of rows
	* @param {int} nCols new number of columns
	* @example m1 = mat([[11,12,13,14],[21,22,23,24],[31,32,33,34],[41,42,43,44]]);
	* m2 = mat([[11,12],[21,22]]);
	* m1.resize(3,3); // m1 = Matrix([[11,12,13],[21,22,23],[31,32,33]])
	* m2.resize(3,3); // m2 = Matrix([[11,12,0],[21,22,0],[0,0,0]])*/
	resize: function(nRows,nCols) {
		if (this.nRows < nRows) {
			for (var r=this.nRows; r<nRows; r++) {
				this.setRow(r,Vector.Zeros(this.nCols));
			}
			this.nRows = nRows;
		}
		if (this.nRows > nRows) {
			this.theArray = this.theArray.slice(0,nRows);
			this.nRows = nRows;
		}
		if (this.nCols < nCols) {
			for (var r=0; r<this.nRows; r++) {
				for (var c=this.nCols; c<nCols; c++) {
					this.theArray[r][c] = 0.;
				}
			}
			this.nCols = nCols;
		}
		if (this.nCols > nCols) {
			for (var r=0; r<this.nRows; r++) {
				this.theArray[r] = this.theArray[r].slice(0,nCols);
			}
			this.nCols = nCols;
		}
	},

	/** Returns resized copy of receiver according to given size (delete extra elements or add zero elements)
	* @param {int} nRows new number of rows
	* @param {int} nCols new number of columns
	* @returns {Matrix} resized copy of receiver
	* @example m1 = mat([[11,12,13],[21,22,23],[31,32,33]]);
	* m2 = m1.resized(2,4); // m2 = Matrix([[11,12,13,0],[21,22,23,0]])
	* m3 = m1.resized(4,2); // m3 = Matrix([[11,12],[21,22],[31,32],[0,0]])*/
	resized: function(nRows,nCols) {
		var ret = this.copy();
		ret.resize(nRows,nCols);
		return ret;
	},

	/** Appends vector(s)/matrix as row(s) to receiver
	* @param {Vector|Matrix|[Vectors]|[Matrices]} rows new row(s) to be appended
	* @example m = mat([[1,2],[3,4]]);
	* m.appendRows(vec([7,6])); // m = Matrix([[1,2],[3,4],[7,6]])
	* m.appendRows(mat([[1,3],[2,4]])); // m = Matrix([[1,2],[3,4],[7,6],[1,3],[2,4]])
	* m.appendRows([vec([2,1]),vec([33,44])]); // m = Matrix([[1,2],[3,4],[7,6],[1,3],[2,4],[2,1],[33,44]])
	* m.appendRows([mat([[2,1],[4,3]]),mat([[99,88],[77,66]])]); // m = Matrix([[1,2],[3,4],[7,6],[1,3],[2,4],[2,1],[33,44],[2,1],[4,3],[99,88],[77,66]])*/
	appendRows: function(rows) {
		if (rows instanceof Vector) {
			if (rows.nItems != this.nCols) { return; }
			this.theArray[this.nRows] = rows.theArray.slice();
			this.nRows++;
			return;
		}
		if (rows instanceof Matrix) {
			if (rows.nCols != this.nCols) { return; }
			for (var r=0; r<rows.nRows; r++) {
				this.theArray[this.nRows] = rows.theArray[r].slice();
				this.nRows++;
			}
			return;
		}
		if (rows instanceof Array) {
			for (var i=0; i<rows.length; i++) { this.appendRows(rows[i]); }
		}
	},

	/** Appends vector(s)/matrix as column(s) to receiver
	* @param {Vector|Matrix|[Vectors]|[Matrices]} rows new row(s) to be appended
	* @example m = mat([[1,2],[3,4]]);
	* m.appendCols(vec([7,6])); // m = Matrix([[1,2,7],[3,4,6]])
	* m.appendCols(mat([[1,3],[2,4]])); // m = Matrix([[1,2,7,1,3],[3,4,6,2,4]])
	* m.appendCols([vec([2,1]),vec([33,44])]); // m = Matrix([[1,2,7,1,3,2,33],[3,4,6,2,4,1,44]])
	* m.appendCols([mat([[2,1],[4,3]]),mat([[99,88],[77,66]])]); // m = Matrix([[1,2,7,1,3,2,33,2,1,99,88],[3,4,6,2,4,1,44,4,3,77,66]])*/
	appendCols: function(cols) {
		if (cols instanceof Vector) {
			if (cols.nItems != this.nRows) { return; }
			for (var r=0; r<this.nRows; r++) { this.theArray[r][this.nCols] = cols.theArray[r]; }
			this.nCols++;
			return;
		}
		if (cols instanceof Matrix) {
			if (cols.nRows != this.nRows) { return; }
			for (var r=0; r<cols.nRows; r++) {
				for (var c=0; c<cols.nCols; c++) { this.theArray[r][this.nCols+c] = cols.theArray[r][c]; }
			}
			this.nCols += cols.nCols;
			return;
		}
		if (cols instanceof Array) {
			for (var i=0; i<cols.length; i++) { this.appendCols(cols[i]); }
		}
	},

	/** Returns row permutation matrix using implicit partial pivoting
	* @param {bool} [saveOrig=true] if true, returns permutated copy of receiver. If false, permutate receiver and return this
	* @Returns {[Matrix,[ints]]} permutated matrix and array containing indices of original matrix according to position in the permutated one ( such that this.rowPermutation(indx) = ret )
	* @example m1 = mat([[1,2,4,9],[1,8,1,1],[9,4,5,3],[1,2,6,2]]);
	* m2 = m1.implicitPartialPivotPermutation();
	* // m2[0] = Matrix([[9,4,5,3],[1,8,1,1],[1,2,6,2],[1,2,4,9]])
	* // m2[1] = [2,1,3,0]
	* // m1 = Matrix([[1,2,4,9],[1,8,1,1],[9,4,5,3],[1,2,6,2]])
	* m3 = m1.rowPermutation(m2[1]); // m3 = Matrix([[9,4,5,3],[1,8,1,1],[1,2,6,2],[1,2,4,9]])
	* m4 = m2[0].rowPermutation(m2[1],true); // m4 = Matrix([[1,2,4,9],[1,8,1,1],[9,4,5,3],[1,2,6,2]])
	* m1 = m1.implicitPartialPivotPermutation(false)[0];
	* // m1 = Matrix([[9,4,5,3],[1,8,1,1],[1,2,6,2],[1,2,4,9]])*/
	implicitPartialPivotPermutation: function(saveOrig) {
		var saveOrig = saveOrig==undefined? true : saveOrig;
		var n = this.nRows;
		var ret = saveOrig? this.copy() : this;
		var vv = [];
		var indx = range(n);
		var big, i, j, k;
		for (i=0; i<n; i++) {
			big = 0.;
			for (j=0; j<n; j++) {
				if ((temp=Math.abs(ret.theArray[i][j])) > big) { big=temp; }
			}
			if (big == 0.0) { return null; }
			vv[i]=1.0/big;
		}
		for (k=0; k<n; k++) {
			big=0.0;
			for (i=k; i<n; i++) {
				temp = vv[i]*Math.abs(ret.theArray[i][k]);
				if (temp > big) {
					big = temp;
					imax = i;
				}
			}
			if (k != imax) {
				ret.swapRows(k,imax);
				vv[imax]=vv[k];
				temp = indx[k]
				indx[k] = imax;
				indx[imax] = temp;
			}
		}
		return [ret,indx];
	},
	
	/** Returns vector x as a solution of this*ret = vec, receiver is assumed to be lower triangular matrix
	* @param {Vector} vec right hand side
	* @param {bool} [saveOrig=true] if true, returns vec will be unchanged and new vector will be returned. If false, solution will be saved to vec and vec will be returned
	* @Returns {Vector} solution of this*ret = vec
	* @example a = mat([[1,0,0,0],[2,3,0,0],[4,5,6,0],[7,8,9,10]]);
	* y = vec([4,17,43,80]);
	* x = a.forwardSubstitution(y);
	* // x = Vector([4,3,2,1])
	* // y = Vector([4,17,43,61])
	* check = a.mul(x); // check = Vector([4,17,43,61])
	* // x = Vector([4,3,2,1])
	* // y = Vector([4,17,43,61])
	* yy - y.copy();
	* x = a.forwardSubstitution(yy,false);
	* // x = Vector([4,3,2,1])
	* // yy = Vector([4,3,2,1])*/
	forwardSubstitution: function(vec,saveOrig) {
		if (!this.isSquare() || !this.canMultiplyVec(vec)) { return null; }
		var saveOrig = saveOrig==undefined? true : saveOrig;
		var temp;
		var n = this.nRows;
		var ret = saveOrig? vec.copy() : vec;
		var i, j;
		for (var i=0; i<n; i++) {
			temp = ret.theArray[i];
			for (var j=0; j<i; j++) {
				temp -= this.theArray[i][j]*ret.theArray[j];
			}
			ret.theArray[i] = temp/this.theArray[i][i];
		}
		return ret;
	},

	/** Returns vector x as a solution of this*ret = vec, receiver is assumed to be upper triangular matrix
	* @param {Vector} vec right hand side
	* @param {bool} [saveOrig=true] if true, returns vec will be unchanged and new vector will be returned. If false, solution will be saved to vec and vec will be returned
	* @Returns {Vector} solution of this*ret = vec
	* @example a = mat([[1,2,3,4],[0,5,6,7],[0,0,8,9],[0,0,0,10]]);
	* y = vec([20,34,25,10]);
	* x = a.forwardSubstitution(y);
	* // x = Vector([4,3,2,1])
	* // y = Vector([20,34,25,10])
	* check = a.mul(x); // check = Vector([20,34,25,10])
	* // x = Vector([4,3,2,1])
	* // y = Vector([20,34,25,10])
	* yy - y.copy();
	* x = a.forwardSubstitution(yy,false);
	* // x = Vector([4,3,2,1])
	* // yy = Vector([4,3,2,1])*/
	backwardSubstitution: function(vec,saveOrig) {
		if (!this.isSquare() || !this.canMultiplyVec(vec)) { return null; }
		var saveOrig = saveOrig==undefined? true : saveOrig;
		var temp;
		var n = this.nRows;
		var ret = saveOrig? vec.copy() : vec;
		var i, j;
		for (i=n-1; i>=0; i--) {
			temp = ret.theArray[i];
			for (j=i+1; j<n; j++) {
				temp -= this.theArray[i][j]*ret.theArray[j];
			}
			ret.theArray[i] = temp/this.theArray[i][i];
		}
		return ret;
	},

	/** Returns vector as a solution of system of linear equations using gaussian elimination method (this * ret = rhs --> ret )
	* @param {Vector} rhs vector of right hand sides
	* @param {int} [pivoting=JSM_PARTPIVOT] what type of pivoting to use. JSM_NOPIVOT stands for no pivoting, JSM_PARTPIVOT for implicit partial pivoting (only row interchanges)
	* @returns {Vector} vector of solution
	* @example a = mat([[1,2,9],[8,3,2],[3,7,3]])
	* b = vec([32,20,26]);
	* x = a.gaussianElimination(b) // x = Vector([1,2,3])
	* x = a.gaussianElimination(b,JSM_PARTPIVOT); // x = Vector([1,2,3])
	* a.gaussianElimination(b,JSM_NOPIVOT,false);
	* // a = Matrix([[1,2,9],[8,-13,-70],[3,1,-29.384615384615387]])
	* // b = Vector([1,2,3])*/
	gaussianElimination: function(rhs,pivoting,saveOrig) {
		if (!this.canMultiplyVec(rhs) || !this.isSquare()) { return null; }
		var pivoting = pivoting || JSM_PARTPIVOT;
		var saveOrig = saveOrig==undefined? true : saveOrig;
		var n = this.nRows;
		var i, j, k;
		if (pivoting == JSM_NOPIVOT) {
			var a = saveOrig? this.copy() : this;
			var b = saveOrig? rhs.copy() : rhs;
			for (k=0; k<n-1; k++) {
				akk = a.theArray[k][k];
				for (i=k+1; i<n; i++) {
					for (j=k+1; j<n; j++) {
						a.theArray[i][j] -= a.theArray[k][j]*a.theArray[i][k]/akk;
					}
					b.theArray[i] -= b.theArray[k]*a.theArray[i][k]/akk;
				}
			}
			return a.backwardSubstitution(b,false);
		}
		if (pivoting == JSM_PARTPIVOT) {
			var a = this.implicitPartialPivotPermutation(saveOrig);
			return a[0].gaussianElimination((saveOrig? rhs.copy() : rhs).itemPermutation(a[1]),JSM_NOPIVOT,false);
		}
	},

	/** Returns matrix, whose columns are solutions of system of equations this*ret = rhs
	* @param {Matrix,[Vectors],Vector} rhs matrix/vector/array of vectors representing right hand sides
	* @param {int} [pivoting=JSM_PARTPIVOT] what type of pivoting to use. JSM_NOPIVOT stands for no pivoting, JSM_PARTPIVOT for implicit partial pivoting (only row interchanges)
	* @param {bool} [saveOrig=true] if true, receiver is not changed. If false, solution will be saved to rhs and rhs will be returned
	* @returns {Matrix} matrix, whose columns are solution for particular right hand sides
	* @example a = mat([[1,2,9],[8,3,2],[3,7,3]]);
	* b1 = vec([32,20,26]);
	* b2 = vec([16,32,26]);
	* b3 = [b1.copy(),b2.copy()];
	* b4 = mat([b1.toArray(),b2.toArray()]);
	* x1 = a.gaussJordanElimination(b1);
	* // x1 = Vector([ 1, 2, 3 ])
	* x2 = a.gaussJordanElimination(b2);
	* // x2 = Vector([ 3.0000000000000018, 2, 1 ])
	* x3 = a.gaussJordanElimination(b3);
	* // x3 = Vector([ 1, 2, 3 ]) ,Vector([ 3.0000000000000018, 2, 1 ])
	* x4 = a.gaussJordanElimination(b4);
	* // x4 = Matrix([[ 1, 3.0000000000000018 ], [ 2, 2 ], [ 3, 1 ]])
	* x5 = a.gaussJordanElimination(b1,JSM_PARTPIVOT);;
	* // x5 = Vector([ 1, 2, 3 ])
	* x6 = a.gaussJordanElimination(b2,JSM_PARTPIVOT);
	* // x6 = Vector([ 3, 2, 1 ])
	* x7 = a.gaussJordanElimination(b3,JSM_PARTPIVOT);
	* // x7 = Vector([ 1, 2, 3 ]) ,Vector([ 3, 2, 1 ])
	* x8 = a.gaussJordanElimination(b4,JSM_PARTPIVOT);
	* // x8 = Matrix([[ 1, 3 ], [ 2, 2 ], [ 3, 1 ]])
	* a.gaussJordanElimination(b4,JSM_NOPIVOT,false);
	* // b4 = Matrix([[ 1, 3.0000000000000018 ], [ 2, 2 ], [ 3, 1 ]])*/
	gaussJordanElimination: function(rhs,pivoting,saveOrig) {
		if (!this.isSquare) { return null; }
		var rhs = rhs || null;
		if (rhs == null) { return null; }
		var rhsInstance = (rhs instanceof Vector)? 1 : ((rhs instanceof Array)? 2 : ((rhs instanceof Matrix)? 3 : 4));
		if (!(rhsInstance==1 || rhsInstance==2 || rhsInstance==3)) { return null; }
		var pivoting = pivoting || JSM_PARTPIVOT;
		var saveOrig = saveOrig==undefined? true : saveOrig;
		var i, j, k, akk;
		var n = this.nRows;
		var nrhs = rhsInstance==2? rhs.length : (rhsInstance==3? rhs.nCols : 0);
		if (pivoting == JSM_NOPIVOT) {
			var a = saveOrig? this.copy() : this;
			var b;
			if (rhsInstance==1 || rhsInstance==3) { b = saveOrig? rhs.copy() : rhs; }
			else {
				b = [];
				for (i=0; i<nrhs; i++) { b[i] = saveOrig? rhs[i].copy() : rhs[i]; }
			}
			for (k=0; k<n; k++) {
				akk = a.theArray[k][k];
				for (i=k+1; i<n; i++) {
					for (j=k+1; j<n; j++) {
						a.theArray[i][j] -= a.theArray[k][j]*a.theArray[i][k]/akk;
					}
					switch (rhsInstance) {
						case 1: b.theArray[i] -= b.theArray[k]*a.theArray[i][k]/akk; break;
						case 2: for (j=0; j<nrhs; j++) { b[j].theArray[i] -= b[j].theArray[k]*a.theArray[i][k]/akk; } break;
						case 3: for (j=0; j<nrhs; j++) { b.theArray[i][j] -= b.theArray[k][j]*a.theArray[i][k]/akk; } break;
					}
				}
			}
			for (k=n-1; k>=0; k--) {
				akk = a.theArray[k][k];
				for (i=k-1; i>=0; i--) {
					for (j=n-1; j>k; j--) {
						a.theArray[i][j] -= a.theArray[k][j]*a.theArray[i][k]/akk;
					}
					switch (rhsInstance) {
						case 1: b.theArray[i] -= b.theArray[k]*a.theArray[i][k]/akk; break;
						case 2: for (j=0; j<nrhs; j++) { b[j].theArray[i] -= b[j].theArray[k]*a.theArray[i][k]/akk; } break;
						case 3: for (j=0; j<nrhs; j++) { b.theArray[i][j] -= b.theArray[k][j]*a.theArray[i][k]/akk; } break;
					}
				}
			}
			for (k=0; k<n; k++) {
				akk = a.theArray[k][k];
				switch (rhsInstance) {
					case 1: b.theArray[k] /= akk; break;
					case 2: for (j=0; j<nrhs; j++) { b[j].theArray[k] /= akk; } break;
					case 3: for (j=0; j<nrhs; j++) { b.theArray[k][j] /= akk; } break;
				}
			}
			return b;
		}
		if (pivoting == JSM_PARTPIVOT) {
			var a = this.implicitPartialPivotPermutation(saveOrig);
			var temp;
			switch (rhsInstance) {
				case 1: temp = saveOrig? rhs.copy() : rhs; temp.permuteItems(a[1]); break;
				case 2:
					temp = [];
					for (i=0; i<nrhs; i++) {
						temp[i] = saveOrig? rhs[i].copy() : rhs[i];
						temp[i].permuteItems(a[1]);
					}
					break;
				case 3:
					temp = saveOrig? rhs.copy() : rhs; temp.permuteRows(a[1]); break;
			}
			return a[0].gaussJordanElimination(temp,JSM_NOPIVOT,false);
		}
	},

	/** Returns lower triangular matrix of Cholesky's (LLT) decomposition of receiver. Receiver must be square, symmetric and positive definite. ( ret * ret^T = this )
	* @returns {Matrix} lower triangular matrix
	* @example a = mat([[1,2,4],[2,13,23],[4,23,77]]);
	* l = a.choleskyDecomposition(); // l = Matrix([[1,0,0],[2,3,0],[4,5,6]])
	* check = l.x(l.T()); // check = Matrix([[1,2,4],[2,13,23],[4,23,77]]);*/
	choleskyDecomposition: function() {
		if (!this.isSquare()) { return null; }
		if (!this.isSymmetric()) { return null; }
		var n = this.nRows;
		var ret = Matrix.Zeros(n,n);
		var retii, temp, i, j, k;
		for (i=0; i<n; i++) {
			retii = this.theArray[i][i]
			for (k=0; k<i; k++) { retii -= ret.theArray[i][k]*ret.theArray[i][k]; }
			if (retii<=0.) { return null; }
			retii = Math.sqrt(retii);
			ret.theArray[i][i] = retii;
			for (j=i; j<n; j++) {
				temp = this.theArray[i][j];
				for (k=0; k<i; k++) { temp -= ret.theArray[i][k]*ret.theArray[j][k]; }
				ret.theArray[j][i] = temp/retii;
			}
		}
		return ret;
	},

	/** Alias for Cholesky decomposition, see {@link Matrix#choleskyDecomposition}*/
	chol: function() { return this.choleskyDecomposition(); },

	/** Alias for Cholesky decomposition, see {@link Matrix#choleskyDecomposition}*/
	lltDecomposition: function() { return this.choleskyDecomposition(); },

	/** Returns lower (L) and upper (U) triangular matrix of the receiver such that L*U=this. Diagonal elements of L are all 1. Receiver must be square.
	* @param {int} [pivoting=JSM_NOPIVOT] what type of pivoting to use. JSM_NOPIVOT stands for no pivoting, JSM_PARTPIVOT for implicit partial pivoting (only row interchanges)
	* @returns {[Matrix,Matrix]} [L,U], lower and upper triangular matrices
	* @example a = mat([[6,5,4],[12,13,10],[18,27,21]]);
	* lu = a.luDecomposition();
	* l = lu[0]; // l = Matrix([[1,0,0],[2,1,0],[3,4,1]])
	* u = lu[1]; // u = Matrix([[6,5,4],[0,3,2],[0,0,1]])
	* check = l.x(u); // check = Matrix([[6,5,4],[12,13,10],[18,27,21]])*/
	luDecomposition: function(pivoting) {
		if (!this.isSquare()) { return null; }
		var pivoting = pivoting || JSM_NOPIVOT;
		var n = this.nRows;
		var l = Matrix.Zeros(n,n);
		var u = Matrix.Zeros(n,n);
		var i, j, k, imax;
		var temp;
		if (pivoting == JSM_NOPIVOT) {
			for (k=0; k<n; k++) {
				l.theArray[k][k] = 1.;
				for (j=k; j<n; j++) {
					temp = this.theArray[k][j];
					for (i=0; i<k; i++) {
						temp -= l.theArray[k][i]*u.theArray[i][j];
					}
					u.theArray[k][j] = temp;
				}
				for (j=k+1; j<n; j++) {
					temp = this.theArray[j][k]
					for (i=0; i<k; i++) {
						temp -= l.theArray[j][i]*u.theArray[i][k]
					}
					l.theArray[j][k] = temp/u.theArray[k][k]
				}
			}
			return [l,u]
		}
		if (pivoting == JSM_PARTPIVOT) {
			var a = this.implicitPartialPivotPermutation()
			var ret = a[0].luDecomposition(JSM_NOPIVOT);
			ret[2] = a[1];
			return ret;
		}
	},

	/** Returns lower triangular (L), diagonal (D) and upper triangular (U) matrix of the receiver such that L*D*U=this. Diagonal elements of both L and U are all 1. Receiver must be square.
	* @param {int} [pivoting=JSM_NOPIVOT] what type of pivoting to use. JSM_NOPIVOT stands for no pivoting, JSM_PARTPIVOT for implicit partial pivoting (only row interchanges)
	* @param {bool} [retDiagAsVector = false] if true, d is returned as vector. If false, d is returned as diagonal matrix
	* @returns {[Matrix,Matrix|Vector,Matrix]} [L,D,U], lower, diagonal and upper triangular matrices
	* @example a = mat([[12,10,8],[24,29,22],[36,66,52]]);
	* ldu = a.lduDecomposition();
	* l = ldu[0]; // l = Matrix([[1,0,0],[2,1,0],[3,4,1]])
	* d = ldu[1]; // d = Matrix([[2,0,0],[0,3,0],[0,0,4]])
	* u = ldu[2]; // u = Matrix([[6,5,4],[0,3,2],[0,0,1]])
	* check = l.x(d).x(u) // check = Matrix([[12,10,8],[24,29,22],[36,66,52]])*/
	lduDecomposition: function(pivoting,retDiagAsVector) {
		if (!this.isSquare()) { return null; }
		var pivoting = pivoting || JSM_NOPIVOT;
		var retDiagAsVector = retDiagAsVector || false;
		if (pivoting == JSM_NOPIVOT) {
			var n = this.nRows;
			var l = zeros(n,n);
			var d = zeros(n);
			var u = zeros(n,n);
			var temp;
			for (var i=0; i<n; i++) {
				l.theArray[i][i] = 1.;
				u.theArray[i][i] = 1.;
				for (var j=0; j<i; j++) {
					temp = this.theArray[i][j];
					for (var k=0; k<j; k++) {
						temp -= l.theArray[i][k]*d.theArray[k]*u.theArray[k][j];
					}
					l.theArray[i][j] = temp/d.theArray[j];
				}
				temp = this.theArray[i][i];
				for (var k=0; k<i; k++) {
					temp -= l.theArray[i][k]*d.theArray[k]*u.theArray[k][i];
				}
				d.theArray[i] = temp;
				for (var j=i+1; j<n; j++) {
					temp = this.theArray[i][j];
					for (var k=0; k<i; k++) {
						temp -= l.theArray[i][k]*d.theArray[k]*u.theArray[k][j];
					}
					u.theArray[i][j] = temp/d.theArray[i];
				}
			}
			if (retDiagAsVector) { return [l,d,u]; }
			return [l,d.toDiagonalMatrix(),u];
		}
		if (pivoting = JSM_PARTPIVOT) {
			var a = this.implicitPartialPivotPermutation();
			var ret = a.lduDecomposition(JSM_NOPIVOT,retDiagAsVector,false);
			ret[3] = a[1];
			return ret;
		}
	},

	/** Returns orthogonal matrix Q^T and upper triangular matrix R of QR decomposition of receiver such that Q*R=this.
	* @param {bool} [saveOrig=true] if true, receiver is not changed. If false, solution (R matrix) will be saved to this and this will be returned
	* @returns {[Matrix,Matrix]} Q^T and R of QR decomposition of receiver
	* @example a = mat([[1,2,9],[8,3,4],[3,7,1]]);
	* qr = a.qrDecomposition();
	* qt = qr[0]; // qt = Matrix([[ -0.1162, -0.9299, -0.3487 ], [ 0.2407, -0.3670, 0.8985 ], [ -0.9636, 0.0205, 0.2665 ]])
	* r = qr[1]; // r = Matrix([[ -8.6023, -5.4636, -5.1148 ], [ 0, 5.6699, 1.5968 ], [ 0, 0, -8.3239 ]])
	* check = q.T().x(r); // check = Matrix([[1,2,9],[8,3,4],[3,7,1]])
	* o = q.T().x(q); // o = Matrix([[1,0,0],[0,1,0],[0,0,1]])*/
	qrDecomposition: function(saveOrig) {
		if (!this.isSquare()) { return null; }
		var saveOrig = saveOrig==undefined? true : saveOrig;
		var n = this.nRows;
		var qt = Matrix.Zeros(n,n);
		var r = saveOrig? this.copy() : this;
		var c = [];
		var d = [];
		var scale, sigma, sum, tau, sing;
		for (k=0; k<n-1; k++) {
			scale=0.0;
			for (i=k;i<n;i++) scale = Math.max(scale, Math.abs(r.theArray[i][k]));
			if (scale == 0.0) { // Singular case.
				sing = true;
				c[k] = 0.0;
				d[k] = 0.0;
			}
			else { // Form Qk and Qk A.
				for (i=k; i<n; i++) { r.theArray[i][k] /= scale; }
				sum = 0.;
				for (i=k; i<n; i++) { sum += (r.theArray[i][k]*r.theArray[i][k]); }
				sigma = Math.sqrt(sum)*(r.theArray[k][k]>=0.? 1. : -1.);
				r.theArray[k][k] += sigma;
				c[k] = sigma*r.theArray[k][k];
				d[k] = -scale*sigma;
				for (j=k+1; j<n; j++) {
					sum = 0.;
					for (i=k; i<n; i++) { sum += r.theArray[i][k]*r.theArray[i][j]; }
					tau = sum/c[k];
					for (i=k; i<n; i++) { r.theArray[i][j] -= tau*r.theArray[i][k]; }
				}
			}
		}
		d[n-1] = r.theArray[n-1][n-1];
		if (d[n-1] == 0.0) { sing = true; }
		for (i=0; i<n; i++) { // Form QT explicitly.
			for (j=0; j<n; j++) { qt.theArray[i][j] = 0.; }
			qt.theArray[i][i] = 1.0;
		}
		for (k=0; k<n-1; k++) {
			if (c[k] != 0.0) {
				for (j=0; j<n; j++) {
					sum = 0.0;
					for (i=k ;i<n; i++) {
						sum += r.theArray[i][k]*qt.theArray[i][j];
					}
					sum /= c[k];
					for (i=k;i<n;i++) {
						qt.theArray[i][j] -= sum*r.theArray[i][k];
					}
				}
			}
		}
		for (i=0; i<n; i++) { // Form R explicitly.
			r.theArray[i][i] = d[i];
			for (j=0; j<i; j++) { r.theArray[i][j] = 0.0; }
		}
		return [qt,r];
	},

	/** Returns inversion of receiver
	* @param {int} [method=JSM_GAUSSJORDAN] method used for solution. Options are JSM_GAUSSJORDAN for Gauss-Jordan method, JSM_LU for solution with LU decomposition
	* @param {[Matrices]} [precompDecomps=undefined] precomputed matrix decompositions if we want to se some
	* @returns {Matrix} inversion of receiver
	* @example a = mat([[2,2,1],[2,3,0.5],[1,0.5,2]]);
	* b = a.inversed(); // b = Matrix([[2.3,-1.4,-0.8],[-1.4,1.2,0.4],[-0.8,0.4,0.8]])
	* check = a.x(b); // check = Matrix([[1,0,0],[0,1,0],[0,0,1]])*/
	inversed: function(method,precompDecomps) {
		if (!this.isSquare) { return null; }
		var method = method || JSM_GAUSSJORDAN;
		switch (method) {
			case JSM_GAUSSJORDAN: return this.gaussJordanElimination(Matrix.Identity(this.nRows,this.nRows));
			default: return null;
		}
	},

	/** Alias for inversion, see {@link Matrix#inversed}*/
	inv: function(method,precompDecomps) { return this.inversed(method||undefined,precompDecomps||undefined); },

	/** Returns determinant of receiver.
	* @param {int} [method=JSM_LU] used method. Options are JSM_LU for using LU decomposition
	* @param {[Matrices]} precompDecomps precomputed matrix decompositions if we want to use some
	* @returns {float} determinant of receiver
	* @example a = mat([[4,2,1],[2,5,3],[1,3,6]]);
	* d = a.determinant(); // d = 67*/
	determinant: function(method,precompDecomps) {
		if (!this.isSquare()) { return null; }
		var method = method || JSM_LU;
		var mats = precompDecomps || undefined
		switch (method) {
			case JSM_LU:
				mats = precompDecomps? precompDecomps : this.luDecomposition();
				var diag = mats[1].diag();
				var ret = diag.theArray[0];
				for (var r=1; r<this.nRows; r++) { ret *= diag.theArray[r]; }
				return ret;
			default: return null;
		}
	},

	/** Alias for determinant, see {@link Matrix#determinant}*/
	det: function(method,precompDecomps) { return this.determinant(method||undefined,precompDecomps||undefined); },

	/** Test receiver's singularity
	* @returns {bool} true if receiver is singular (if abs(this.det()) < JSM_TOL), false otherwise
	* @example a = mat([[1,2,3],[2,4,6],[4,5,6]]);
	* b = mat([[4,2,1],[2,5,3],[1,3,6]]);
	* s1 = a.isSingular(); // s1 = true
	* s2 = b.isSingular(); // s1 = false*/
	isSingular: function() {
		var d = this.determinant();
		return isNaN(d)? true : (Math.abs(d) < JSM_TOL);
	},

	/** Returns vector/array of vectors/matrix as a solution of system of linear equations (this*ret=rhs --> ret). Receiver must be square. TODO: solution with LU decomposition with permutation
	* @param {Vector|[Vectors]|Matrix} rhs vector of right hand sides. Array of vectors or Matrix as rhs is supported only with JSM_GAUSSJORDAN method
	* @param {int} [method=JSM_GAUSS] method of the solution. Implemented methods are JSM_GAUSS for gaussian elimination, JSM_GAUSSJORDAN for Gauss-Jordan elimination (supporting more than one right hand sides in form of array of vectors or matrix with columns as rhs), JSM_LU for using LU decomposition, JSM_LDU for using LDU decomposition, JSM_CHOLESKY for using Cholesky decomposition
	* @param {bool} [saveOrig=true] if true, receiver and rhs is not changed. If false, solution will be saved to rhs and rhs will be returned and receiver will be changed
	* @param {[Matrices]} [precompDecomps=undefined] array of Matrices objects, used for solution using matrix decomposition as precomputed values (the decomposition is performed within solveForRhs function if args parameter is not specified)
	* @returns {Vector|[Vectors]|Matrix} vector/array of vectors/matrix of solution
	* @example a = mat([[1,2,9],[8,3,2],[3,7,3]]);
	* b = vec([32,20,26]);
	* x1 = a.solveForRhs(b); // x1 = Vector([1,2,3])
	* x2 = a.solveForRhs(b,JSM_GAUSS); // x2 = Vector([1,2,3])
	* x3 = a.solveForRhs(b,JSM_GAUSSJORDAN); // x3 = Vector([1,2,3])
	* x4 = a.solveForRhs(b,JSM_LU); // x4 = Vector([1,2,3])
	* x5 = a.solveForRhs(b,JSM_LDU); // x5 = Vector([1,2,3])
	* x6 = a.solveForRhs(b,JSM_CHOLESKY); // x6 = Vector([1,2,3])
	* x7 = a.solveForRhs(b,JSM_QR); // x7 = Vector([1,2,3])*/
	solveForRhs: function(rhs,method,saveOrig,precompDecomps) {
		var saveOrig = saveOrig==undefined? true : saveOrig;
		var method = method || JSM_GAUSS;
		if (method==JSM_GAUSS) { // gaussian elimination
			return this.gaussianElimination(rhs,JSM_PARTPIVOT,saveOrig);
		}
		if (method==JSM_GAUSSJORDAN) { // gaussian elimination
			return this.gaussJordanElimination(rhs,JSM_PARTPIVOT,saveOrig);
		}
		if (method==JSM_LU) { // use LU factorization
			var lu = precompDecomps || this.luDecomposition();
			var l = lu[0];
			var u = lu[1];
			var y;
			if (rhs instanceof Vector) {
				y = l.forwardSubstitution(rhs,saveOrig);
				return u.backwardSubstitution(y,false);
			}
			if (rhs instanceof Matrix) {
				var ret = saveOrig? Matrix.Zeros(this.nRows,rhs.nCols) : rhs;
				for (var i=0; i<rhs.nCols; i++) {
					y = l.forwardSubstitution(rhs.getCol(i),false);
					ret.setCol(i,u.backwardSubstitution(y,false));
				}
				return ret;
			}
			if (rhs instanceof Array) {
				return null;
				// TODO
			}
		}
		if (method==JSM_LDU) { // use LDU decomposition
			var ldu = precompDecomps || this.lduDecomposition(JSM_NOPIVOT,true);
			var l = ldu[0];
			var d = (ldu[1] instanceof Matrix)? ldu[1].diag() : ldu[1];
			var u = ldu[2];
			var y = l.forwardSubstitution(rhs,saveOrig);
			for (var r=0; r<this.nRows; r++) { y.theArray[r] /= d.theArray[r]; }
			return u.backwardSubstitution(y,false);
		}
		if (method==JSM_CHOLESKY) { // use Cholesky's decomposition
			var l = precompDecomps || this.choleskyDecomposition();
			if (l==null) { return null; }
			var y = l.forwardSubstitution(rhs,saveOrig);
			return l.T().backwardSubstitution(y,false);
		}
		if (method==JSM_QR) { // use QR decomposition
			var qr = precompDecomps || this.qrDecomposition(saveOrig);
			return qr[1].backwardSubstitution(qr[0].mulv(rhs),false);
		}
		return null;
	},
	/**#@-*/
}

/** Constructs new Matrix from given 2D array
* @param {[floats](2D)} [arry=[]] array containing elements of new matrix
* @returns {Matrix} new Matrix object
* @example m = Matrix.create([[1,2,3],[4,5,6],[7,8,9]]) // m = Matrix([[1,2,3],[4,5,6],[7,8,9]])*/
Matrix.create = function(arry) {
	var a = (arry==null)? [[]]:arry;
	ret = new Matrix();
	ret.fromArray(a);
	return ret;
}

/** Constructs new diagonal matrix from given 1D array
* @param {[floats]|Vector} arry array or vector containing elements of new matrix
* @returns {Matrix} new Matrix object 
* @example m1 = Matrix.Diagonal([1,2,3]); // m1 = Matrix([[1,0,0],[0,2,0],[0,0,3]])
* m2 = Matrix.Diagonal(vec([1,2,3])); // m2 = Matrix([[1,0,0],[0,2,0],[0,0,3]])*/
Matrix.Diagonal = function(arry) {
	return Vector.create(arry.theArray || arry).diag();
}

/** Constructs new identity matrix of given size
* @param {int} size size of returned matrix
* @returns {Matrix} identity matri
* @example m = Matrix.Identity(4) // m = Matrix([[1,0,0,0],[0,1,0,0],[0,0,1,0],[0,0,0,1]])*/
Matrix.Identity = function(size) {
	var a = []
	for (var i=0; i<size; i++) { a[i] = 1.; }
	return Matrix.Diagonal(a);
}

/** Alias for Matrix.Identity(), see {@link Matrix#Identity}*/
Matrix.I = function(nRows) { return Matrix.Identity(nRows); }

/** Creates a matrix of given size full of zeros
* @param {int} [nRows=0] number of rows
* @param {int} [nCols=0] number of columns
* @returns {Matrix} new matrix full of zero
* @example m = Matrix.Zeros(2,3); // m = Matrix([[0,0,0],[0,0,0]])*/
Matrix.Zeros = function(nRows,nCols) {
	var nRows = nRows || 0;
	var nCols = nCols || 0;
	var a = []
	for (var r=0; r<nRows; r++) {
		a[r] = []
		for (var c=0; c<nCols; c++) {
			a[r][c] = 0.;
		}
	}
	return Matrix.create(a);
}

/** Creates a matrix of given size full of ones
* @param {int} [nRows=0] number of rows
* @param {int} [nCols=0] number of columns
* @returns {Matrix} new vector full of zero
* @example m = Matrix.Ones(2,3); // m = Matrix([[1,1,1],[1,1,1]])*/
Matrix.Ones = function(nRows,nCols) {
	var nRows = nRows || 0;
	var nCols = nCols || 0;
	var a = []
	for (var r=0; r<nRows; r++) {
		a[r] = []
		for (var c=0; c<nCols; c++) {
			a[r][c] = 1.;
		}
	}
	return Matrix.create(a);
}



/* **************************************************
*
* Initializing functions
*
***************************************************/

/** Returns new Matrix object from given array
* @param {[floats](2D)} [arry=[]] array containing matrix elements
* @returns {Matrix} new Matrix object
* @example m = mat([[1,2,3],[4,5,6],[7,8,9]]) // m = Matrix([[1,2,3],[4,5,6],[7,8,9]])*/
mat = function(arry) {
	var arry = (arry==null)? [[]]:arry;
	return Matrix.create(arry);
}

/** Returns new Vector object from given array
* @param {[floats]} [arry=[]] array containing vector elements
* @returns {Vector} new Vector object
* @example v = vec([1,2,3,4]) // v = Vector([1,2,3,4])*/
vec = function(arry) {
	var arry = (arry==null)? []:arry;
	return Vector.create(arry);
}

/** Returns new Vector or Matrix full of given size full of zeros
* @param {int} nRows number of rows of returned object
* @param {int} [nCols] if specified, new Matrix of size (nRows,nCols) is returned, new vector of size nRows otherwise
* @returns {Matrix|Vector} new Matrix or Vector object
* @example m = zeros(2,3); // m = Matrix([[0,0,0],[0,0,0]])
* v = zeros(4) // v = Vector([0,0,0,0])*/
zeros = function(nRows,nCols) {
	if (nCols) { return Matrix.Zeros(nRows,nCols); }
	if (nRows) { return Vector.Zeros(nRows); }
	return null;
}

/** Returns new Vector or Matrix full of given size full of ones
* @param {int} nRows number of rows of returned object
* @param {int} [nCols] if specified, new Matrix of size (nRows,nCols) is returned, new vector of size nRows otherwise
* @returns {Matrix|Vector} new Matrix or Vector object
* @example m = zeros(2,3); // m = Matrix([[1,1,1],[1,1,1]])
* v = zeros(4) // v = Vector([1,1,1,1])*/
ones = function(nRows,nCols) {
	if (nCols) { return Matrix.Ones(nRows,nCols); }
	if (nRows) { return Vector.Ones(nRows); }
	return null;
}

/** Returned identity matrix of given size, see {@link Matrix#Identity}
* @param {int} nRows number of rows and columns of returned matrix
* @returns {Matrix} nRows x nRows identity matrix
* @example m = identity(3); // m = Matrix([[1,0,0],[0,1,0],[0,0,1]])*/
identity = function(nRows) { return Matrix.Identity(nRows); }

/** Alias for dentity(), see {@link identity}*/
eye = function(nRows) { return Matrix.Identity(nRows); }


/* ***********************************************************
*
* Other functions
*
************************************************************/

/** Linear system equation solver. Returns vector x as a solution of a*ret=rhs, see {@link Matrix#solveForRhs} for input desription
* @param {Matrix} a
* @param {Vector|[Vectors]|Matrix} rhs
* @param {bool} [saveOrig=true]
* @param {int} [method=JSM_GAUSS]
* @param {[Matrices]} [precompDecomps]
* @returns {Vector|[Vectors]|Matrix} solution x of a*x=rhs*/
linSolve = function(a,rhs,saveOrig,method,precompDecomps) { return a.solveForRhs(rhs,saveOrig||undefined,method||undefined,precompDecomps||undefined); }


/** Eigenvalues and eigenvectors solver. Returns first nEigModes eigenvales and eigenvectors of problem K*vi=oi^2*M*vi (oi is i-th eigenvale and vi is i-th eigenvector). Typycal example of usage is eigenvale dynamics.
* @param {Matrix} K first matrix (stiffness matrix for dynamics)
* @param {Matrix} M second matrix (mass matrix for dynamics)
* @param {int} [nEigModes=1] number of first eigenvalues and eigenvectors to be returned
* @param {int} [method=JSM_INV_IT] method of the solution. JSM_INV_IT stands for Stodola's inverse iterations methods with Gramm-Schmidt orthogonalization
* @param {int} [maxiter=1000] maximum number of iterations
* @returns {[[floats],[Vectors]]} [[o1,o2,...,oN],[v1,v2,...,vN]], oi is i-th eigenvale (squared eigenfrequency for dynamics), vi is i-th eigenvector. N = nEigModes
* @example K = mat([[9,3,0,0],[3,8,2,0],[0,2,7,2],[0,0,2,8]]);
* M = Matrix.Diagonal([3,2,4,3]);
* e = eigSolve(K,M,4);
* o = e[0];
* v = e[1];
* o1 = o[0]; // o1 = 1.2504038497315775
* o2 = o[1]; // o2 = 2.279120228657416
* o3 = o[2]; // o3 = 2.948867853467429
* o4 = o[3]; // o4 = 4.938274734810245
* v1 = v[0]; // v1 = Vector([ 0.1288306767139194, -0.22540165581364102, 0.4265175098731745, -0.20077135620035116 ])
* v2 = v[1]; // v2 = Vector([ 0.4514687335612619, -0.32545467450354726, -0.11713473474653795, 0.2014979513549984 ])
* v3 = v[2]; // v3 = Vector([ 0.14681878659487543, -0.007507144503266177, -0.21233717286242818, -0.5016212772448967 ])
* v4 = v[3]; // v4 = Vector([ 0.3022519091588433, 0.585847242190032, 0.09630780190740544, 0.028264211960396433 ])
* c1 = K.x(v1).sub(M.x(v1).x(o1)); // c1 = Vector([ 6.045241529584189e-10, -2.9052349415081835e-10, -2.091367079515294e-10, 2.697864154299623e-10 ])
* c2 = K.x(v2).sub(M.x(v2).x(o2)); // c2 = Vector([ 8.74326433475403e-9, -4.933693453779142e-10, -1.5765841965276195e-8, -2.9551703084607084e-8 ])
* c3 = K.x(v3).sub(M.x(v3).x(o3)); // c3 = Vector([ 4.5619911848149286e-8, 1.2227673172604536e-8, -9.32639299122684e-10, 1.3564216416739328e-8 ])
* c4 = K.x(v4).sub(M.x(v4).x(o4)); // c4 = Vector([ 9.357854047209457e-9, -3.189910557921394e-10, -1.804510585401431e-8, -3.197205944438508e-8 ])*/
eigSolve = function(K,M,nEigModes,method,maxiter) {
	var K = K;
	var M = M;
	if (!K.isSquare() || !M.isSquare() || !K.isSameSizeAs(M)) { return null; }
	var nEigModes = nEigModes || 1; // nEigModes first eigenvalues and eigenvectors will be returned
	if (nEigModes > K.nRows) { nEigModes = K.nRows; }
	var method = method || JSM_INV_IT;
	var maxiter = maxiter || 1000;
	if (method == JSM_INV_IT) { // Stodola's inverse iteration with Gramm-Schmidt orthogonalization
		var DM = K.inv().mulm(M);
		var tol = 1e-32;
		var eigVals = [];
		var eigVecs = [];
		var eigVecsMulmM = [];
		var rho, rhoRatio, rhoNew, iter;
		var xk = Vector.Zeros(K.nRows);
		for (var k=0; k<nEigModes; k++) {
			xk.zero(); xk.set(0,1.);
			iter = 0;
			rhoNew = 0.;
			do {
				xk = DM.mulv(xk);
				for (var j=0; j<k; j++) { xk.isub(eigVecs[j].mulf(eigVecsMulmM[j].dot(xk))); } // Gramm-Schmidt orthogonalization
				xk.imulf(1./Math.sqrt(xk.mulm(M).dot(xk)));
				rho = rhoNew;
				rhoNew = (xk.mulm(K).dot(xk)) / (xk.mulm(M).dot(xk));
				rhoRatio = Math.abs(rhoNew-rho)/rhoNew;
				iter++;
				if (iter > maxiter) {
					if (rhoRatio > JSM_TOL) {
						//alert('Eigenvalue solution failed...');
					}
					break;
				}
			} while (rhoRatio > tol)
			eigVals.push(rhoNew);
			eigVecs.push(xk.copy());
			eigVecsMulmM.push(xk.mulm(M));
		}
		return [eigVals,eigVecs];
	}
	if (method == JSM_SUBSPACE) { // subspace iteration
		var n = K.nRows;
		var kk, mm, r, ret, q;
		x = Matrix.Ones(n,nEigModes);
		for (var i=0; i<n; i++) { x.set(i,i,0.); }
		lu = K.luDecomposition()
		for (var i=0; i<50; i++) {
			x = K.solveForRhs(M.x(x),JSM_LU,false,lu);
			kk = x.T().x(K).x(x);
			mm = x.T().x(M).x(x);
			r = eigSolve(kk,mm,nEigModes);
			q = Matrix.Zeros(nEigModes,nEigModes);
			for (var j=0; j<nEigModes; j++) {
				q.setCol(j,r[1][j]);
			}
			x = x.x(q);
		}
		ret = [[],[]];
		for (var i=0; i<nEigModes; i++) {
			ret[0].push(r[0][i]);
			ret[1].push(x.getCol(i));
		}
		return ret;
	}
}

/** Returns sequence (array) with start, stop, step. Inspired by <a href="www.python.org">Python</a> syntax
* @param {float|int} a start/stop (according to b, see example)
* @param {float|int} [b] stop
* @param {float|int} [c=1] step
* @returns {[floats]} array containing given sequence
* @example r = range(8);     // r = [0,1,2,3,4,5,6,7]
* r = range(2,8);   // r = [2,3,4,5,7]
* r = range(2,8,2); // r = [2,4,6]*/
range = function(a,b,c) {
	var start;
	var step;
	var stop;
	if (b==null && c==null) {
		start = 0;
		stop  = a;
		step  = 1;
	} else if (c==null) {
		start = a;
		stop  = b;
		step  = 1;
	} else {
		start = a;
		stop  = b;
		step  = c;
	}
	var ret = []
	var item = start;
	while (item < stop) {
		ret.push(item);
		item += step;
	}
	return ret;
}

/**@ignore*/
Array.prototype.indexOf = function(val) {
	for (var i=0; i<this.length; i++) {
		if (this[i] === val) { return i; }
	}
}
